@@ -47,10 +47,10 @@
 
 #include "subagent.h"
 
-netsnmp_feature_child_of(agentx_subagent, agentx_all)
-netsnmp_feature_child_of(agentx_enable_subagent, agentx_subagent)
+netsnmp_feature_child_of(agentx_subagent, agentx_all);
+netsnmp_feature_child_of(agentx_enable_subagent, agentx_subagent);
 
-netsnmp_feature_require(remove_trap_session)
+netsnmp_feature_require(remove_trap_session);
 
 #ifdef USING_AGENTX_SUBAGENT_MODULE
 
@@ -247,6 +247,8 @@ static void
 send_agentx_error(netsnmp_session *session, netsnmp_pdu *pdu, int errstat, int errindex)
 {
     pdu = snmp_clone_pdu(pdu);
+    if (!pdu)
+        return;
     pdu->command   = AGENTX_MSG_RESPONSE;
     pdu->version   = session->version;
     pdu->errstat   = errstat;
@@ -503,6 +505,9 @@ handle_agentx_packet(int operation, netsnmp_session * session, int reqid,
      */
 
     internal_pdu = snmp_clone_pdu(pdu);
+    if (!internal_pdu)
+        return 1;
+    free(internal_pdu->contextName);
     internal_pdu->contextName = (char *) internal_pdu->community;
     internal_pdu->contextNameLen = internal_pdu->community_len;
     internal_pdu->community = NULL;
@@ -548,6 +553,8 @@ handle_subagent_response(int op, netsnmp_session * session, int reqid,
     }
 
     pdu = snmp_clone_pdu(pdu);
+    if (!pdu)
+        return 1;
     DEBUGMSGTL(("agentx/subagent",
                 "handling AgentX response (cmd 0x%02x orig_cmd 0x%02x)"
                 " (req=0x%x,trans=0x%x,sess=0x%x)\n",
@@ -642,6 +649,8 @@ handle_subagent_set_response(int op, netsnmp_session * session, int reqid,
                 (unsigned)pdu->command, (unsigned)pdu->reqid,
 		(unsigned)pdu->transid, (unsigned)pdu->sessid));
     pdu = snmp_clone_pdu(pdu);
+    if (!pdu)
+        return 1;
 
     asi = (struct agent_netsnmp_set_info *) magic;
     retsess = asi->sess;
@@ -675,11 +684,13 @@ handle_subagent_set_response(int op, netsnmp_session * session, int reqid,
         pdu->variables = NULL;  /* the variables were added by us */
     }
 
-    netsnmp_assert(retsess != NULL);
-    pdu->command = AGENTX_MSG_RESPONSE;
-    pdu->version = retsess->version;
+    if (retsess && pdu) {
+        pdu->command = AGENTX_MSG_RESPONSE;
+        pdu->version = retsess->version;
 
-    if (!snmp_send(retsess, pdu)) {
+        if (!snmp_send(retsess, pdu))
+            snmp_free_pdu(pdu);
+    } else if (pdu) {
         snmp_free_pdu(pdu);
     }
     DEBUGMSGTL(("agentx/subagent", "  FINISHED\n"));
@@ -870,6 +881,8 @@ subagent_open_master_session(void)
                       agentx_realloc_build, agentx_check_packet, NULL);
 
     if (main_session == NULL) {
+        /* snmp_add_full() frees 't' upon failure. */
+        t = NULL;
         if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
                                     NETSNMP_DS_AGENT_NO_CONNECTION_WARNINGS)) {
             char buf[1024];
