@@ -1,9 +1,17 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/hardware/cpu.h>
+#include "cpu.h"
+
+netsnmp_feature_child_of(hardware_cpu, libnetsnmpmibs);
+
+netsnmp_feature_child_of(hardware_cpu_copy_stats, hardware_cpu);
+netsnmp_feature_child_of(hardware_cpu_load, hardware_cpu);
+netsnmp_feature_child_of(hardware_cpu_get_cache, hardware_cpu);
+netsnmp_feature_child_of(hardware_cpu_get_byName, hardware_cpu);
 
-extern NetsnmpCacheLoad netsnmp_cpu_arch_load;
 static void _cpu_update_stats( unsigned int, void* );
 
 static int _cpuAutoUpdate =  5;
@@ -107,14 +115,14 @@ netsnmp_cpu_info *netsnmp_cpu_get_byIdx(  int idx, int create ) {
             return cpu;
         }
     }
-    if (cpu)
-        SNMP_FREE(cpu); /* just in case */
+    SNMP_FREE(cpu); /* just in case */
     return NULL;  /* Shouldn't happen! */
 }
 
     /*
      * Work with a list of CPU entries, indexed by name
      */
+#ifndef NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_GET_BYNAME
 netsnmp_cpu_info *netsnmp_cpu_get_byName( char *name, int create ) {
     netsnmp_cpu_info *cpu;
 
@@ -134,7 +142,13 @@ netsnmp_cpu_info *netsnmp_cpu_get_byName( char *name, int create ) {
     cpu = SNMP_MALLOC_TYPEDEF( netsnmp_cpu_info );
     if (!cpu)
         return NULL;
-    strcpy(cpu->name, name);
+    if (strlen(name) >= sizeof(cpu->name)) {
+        free(cpu);
+        snmp_log(LOG_ERR, "Name of CPU is too large: %s\n", name);
+        return NULL;
+    }
+
+    strlcpy(cpu->name, name, sizeof(cpu));
     if ( _cpu_tail ) {
         cpu->idx = _cpu_tail->idx+1;
         _cpu_tail->next = cpu;
@@ -146,11 +160,15 @@ netsnmp_cpu_info *netsnmp_cpu_get_byName( char *name, int create ) {
     }
     return cpu;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_GET_BYNAME */
 
+#ifndef NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_GET_CACHE
 netsnmp_cache *netsnmp_cpu_get_cache( void ) {
     return _cpu_cache;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_GET_CACHE */
 
+#ifndef NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_LOAD
 int netsnmp_cpu_load( void ) {
         /*
          * If we're automatically updating the stats regularly,
@@ -159,6 +177,7 @@ int netsnmp_cpu_load( void ) {
     return ( _cpuAutoUpdate ? 1
                             : netsnmp_cache_check_and_reload( _cpu_cache ));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_LOAD */
 
     /*
      * Call the system-specific load routine regularly,
@@ -214,10 +233,14 @@ _cpu_update_stats( unsigned int reg, void* magic ) {
                            cpu->wait_ticks +
                            cpu->kern_ticks +
                            cpu->intrpt_ticks +
-                           cpu->sirq_ticks;
+                           cpu->sirq_ticks +
+                           cpu->steal_ticks +
+                           cpu->guest_ticks +
+                           cpu->guestnice_ticks;
     }
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_COPY_STATS
 void _cpu_copy_stats( netsnmp_cpu_info *cpu )
 {
     netsnmp_cpu_info *cpu2;
@@ -237,6 +260,9 @@ void _cpu_copy_stats( netsnmp_cpu_info *cpu )
     cpu2->kern_ticks = cpu->kern_ticks;
     cpu2->intrpt_ticks = cpu->intrpt_ticks;
     cpu2->sirq_ticks = cpu->sirq_ticks;
+    cpu2->steal_ticks = cpu->steal_ticks;
+    cpu2->guest_ticks = cpu->guest_ticks;
+    cpu2->guestnice_ticks = cpu->guestnice_ticks;
 
     cpu2->nInterrupts  = cpu->nInterrupts;
     cpu2->nCtxSwitches = cpu->nCtxSwitches;
@@ -245,3 +271,4 @@ void _cpu_copy_stats( netsnmp_cpu_info *cpu )
     cpu2->pageIn     = cpu->pageIn;
     cpu2->pageOut    = cpu->pageOut;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_HARDWARE_CPU_COPY_STATS */
