@@ -3,7 +3,7 @@ config_require(hardware/memory/hw_mem)
 #if defined(linux)
 config_require(hardware/memory/memory_linux)
 
-#elif (defined(darwin7) || defined(darwin8) || defined(darwin9))
+#elif (defined(darwin))
 config_require(hardware/memory/memory_darwin)
 
 #elif (defined(freebsd2) || defined(freebsd3) || defined(freebsd4)  || defined(freebsd5)|| defined(freebsd6))
