@@ -9,13 +9,9 @@
 /*
  * Try to use an initial size that will cover default cases. We aren't talking
  * about huge files, so why fiddle about with reallocs?
- * I checked /proc/meminfo sizes on 3 different systems: 598, 644, 654
- * 
- * On newer systems, the size is up to around 930 (2.6.27 kernel)
- *   or 1160  (2.6.28 kernel)
  */
-#define MEMINFO_INIT_SIZE   1279
-#define MEMINFO_STEP_SIZE   256
+#define MEMINFO_INIT_SIZE   4096
+#define MEMINFO_STEP_SIZE   4096
 #define MEMINFO_FILE   "/proc/meminfo"
 
     /*
@@ -29,7 +25,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
     ssize_t      bytes_read;
     char        *b;
     unsigned long memtotal = 0,  memfree = 0, memshared = 0,
-                  buffers = 0,   cached = 0,
+                  buffers = 0,   cached = 0, sreclaimable = 0,
                   swaptotal = 0, swapfree = 0;
 
     netsnmp_memory_info *mem;
@@ -46,6 +42,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         buff = (char*)malloc(bsize+1);
         if (NULL == buff) {
             snmp_log(LOG_ERR, "malloc failed\n");
+            close(statfd);
             return -1;
         }
     }
@@ -53,6 +50,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         b = (char*)realloc(buff, bsize + MEMINFO_STEP_SIZE + 1);
         if (NULL == b) {
             snmp_log(LOG_ERR, "malloc failed\n");
+            close(statfd);
             return -1;
         }
         buff = b;
@@ -68,6 +66,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
     close(statfd);
     if (bytes_read <= 0) {
         snmp_log_perror(MEMINFO_FILE);
+        buff[0] = '\0';
     } else {
         buff[bytes_read] = '\0';
     }
@@ -89,13 +88,19 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (first)
             snmp_log(LOG_ERR, "No MemFree line in /proc/meminfo\n");
     }
-    b = strstr(buff, "MemShared: ");
-    if (b)
-        sscanf(b, "MemShared: %lu", &memshared);
+    if (0 == netsnmp_os_prematch("Linux","2.4")) {
+        b = strstr(buff, "MemShared: ");
+        if (b)
+            sscanf(b, "MemShared: %lu", &memshared);
+        else if (first)
+            snmp_log(LOG_ERR, "No MemShared line in /proc/meminfo\n");
+    }
     else {
-        if (first)
-            if (0 == netsnmp_os_prematch("Linux","2.4"))
-                snmp_log(LOG_ERR, "No MemShared line in /proc/meminfo\n");
+        b = strstr(buff, "Shmem: ");
+        if (b)
+            sscanf(b, "Shmem: %lu", &memshared);
+        else if (first)
+            snmp_log(LOG_ERR, "No Shmem line in /proc/meminfo\n");
     }
     b = strstr(buff, "Buffers: ");
     if (b)
@@ -125,6 +130,9 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (first)
             snmp_log(LOG_ERR, "No SwapFree line in /proc/meminfo\n");
     }
+    b = strstr(buff, "SReclaimable: ");
+    if (b)
+        sscanf(b, "SReclaimable: %lu", &sreclaimable);
     first = 0;
 
 
@@ -155,8 +163,6 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         mem->other = -1;
     }
 
-   /* Shared memory is not reported by Linux 2.6 kernel */
-   if (0 != netsnmp_os_prematch("Linux","2.6")) {
     mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_SHARED, 1 );
     if (!mem) {
         snmp_log_perror("No Shared Memory info entry");
@@ -165,10 +171,9 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
              mem->descr = strdup("Shared memory");
         mem->units = 1024;
         mem->size  = memshared;
-        mem->free  = -1;
+        mem->free  = 0;    /* All in use */
         mem->other = -1;
     }
-   }
 
     mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_CACHED, 1 );
     if (!mem) {
@@ -177,7 +182,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (!mem->descr)
              mem->descr = strdup("Cached memory");
         mem->units = 1024;
-        mem->size  = cached;
+        mem->size  = cached + sreclaimable;
         mem->free  = 0;     /* Report cached size/used as equal */
         mem->other = -1;
     }
