@@ -64,9 +64,13 @@
 #include <sys/statfs.h>
 #endif
 
-netsnmp_feature_require(se_find_free_value_in_slist)
-netsnmp_feature_require(date_n_time)
-netsnmp_feature_require(ctime_to_timet)
+netsnmp_feature_require(se_find_free_value_in_slist);
+netsnmp_feature_require(date_n_time);
+netsnmp_feature_require(ctime_to_timet);
+
+#ifndef MNTTYPE_AUTOFS
+#define MNTTYPE_AUTOFS	"autofs"
+#endif
 
 #if defined(bsdi4) || defined(freebsd3) || defined(freebsd4) || defined(freebsd5) || defined(darwin)
 #if HAVE_GETFSSTAT && defined(MFSNAMELEN)
@@ -664,7 +668,7 @@ Init_HR_FileSys(void)
 #endif
 }
 
-const char     *HRFS_ignores[] = {
+static const char *HRFS_ignores[] = {
 #ifdef MNTTYPE_IGNORE
     MNTTYPE_IGNORE,
 #endif
@@ -677,13 +681,8 @@ const char     *HRFS_ignores[] = {
 #ifdef MNTTYPE_PROCFS
     MNTTYPE_PROCFS,
 #endif
-#ifdef MNTTYPE_AUTOFS
     MNTTYPE_AUTOFS,
-#else
-    "autofs",
-#endif
 #ifdef linux
-    "autofs",
     "bdev",
     "binfmt_misc",
     "cpuset",
@@ -705,7 +704,6 @@ const char     *HRFS_ignores[] = {
     "shm",
     "sockfs",
     "sysfs",
-    "tmpfs",
     "usbdevfs",
     "usbfs",
 #endif
@@ -720,6 +718,7 @@ const char     *HRFS_ignores[] = {
 int
 Get_Next_HR_FileSys(void)
 {
+next:
 #if HAVE_GETFSSTAT
     if (HRFS_index >= fscount)
         return -1;
@@ -746,8 +745,7 @@ Get_Next_HR_FileSys(void)
         case MNT_PROCFS:
 #endif
         case MNT_SFS:
-            return Get_Next_HR_FileSys();
-            break;
+            goto next;
     }
     return HRFS_index++;
 #else
@@ -767,7 +765,7 @@ Get_Next_HR_FileSys(void)
 
     for (cpp = HRFS_ignores; *cpp != NULL; ++cpp)
         if (!strcmp(HRFS_entry->HRFS_type, *cpp))
-            return Get_Next_HR_FileSys();
+            goto next;
 
     /*
      * Try and ensure that index values are persistent
@@ -834,6 +832,17 @@ Check_HR_FileSys_NFS (void)
     return 0;		/* no NFS file system */
 }
 
+/* This function checks whether current file system is an AutoFs
+ * HRFS_entry must be valid prior to calling this function
+ * return 1 if AutoFs, 0 otherwise
+ */
+int
+Check_HR_FileSys_AutoFs(void)
+{
+    return HRFS_entry->HRFS_type &&
+        strcmp(HRFS_entry->HRFS_type, MNTTYPE_AUTOFS) == 0;
+}
+
 void
 End_HR_FileSys(void)
 {
