@@ -283,33 +283,23 @@ void
 init_hr_swrun(void)
 {
 #ifdef cygwin
-    OSVERSIONINFO   ver;
     HMODULE         h;
 
-    memset(&ver, 0, sizeof ver);
-    ver.dwOSVersionInfoSize = sizeof ver;
-    GetVersionEx(&ver);
-
-    if (ver.dwPlatformId == VER_PLATFORM_WIN32_NT) {
-        h = LoadLibrary("psapi.dll");
-        if (h) {
-            myEnumProcessModules =
-                (ENUMPROCESSMODULES) GetProcAddress(h,
-                                                    "EnumProcessModules");
-            myGetModuleFileNameEx =
-                (GETMODULEFILENAME) GetProcAddress(h,
-                                                   "GetModuleFileNameExA");
-            myGetProcessMemoryInfo =
-                (GETPROCESSMEMORYINFO) GetProcAddress(h,
-                                                      "GetProcessMemoryInfo");
-            if (myEnumProcessModules && myGetModuleFileNameEx)
-                query = CW_GETPINFO_FULL;
-            else
-                snmp_log(LOG_ERR, "hr_swrun failed NT init\n");
-        } else
-            snmp_log(LOG_ERR, "hr_swrun failed to load psapi.dll\n");
-    } else {
-        h = GetModuleHandle("KERNEL32.DLL");
+    if ((h = LoadLibrary("psapi.dll")) != NULL) {
+        myEnumProcessModules =
+            (ENUMPROCESSMODULES) GetProcAddress(h,
+                                                "EnumProcessModules");
+        myGetModuleFileNameEx =
+            (GETMODULEFILENAME) GetProcAddress(h,
+                                               "GetModuleFileNameExA");
+        myGetProcessMemoryInfo =
+            (GETPROCESSMEMORYINFO) GetProcAddress(h,
+                                                  "GetProcessMemoryInfo");
+        if (myEnumProcessModules && myGetModuleFileNameEx)
+            query = CW_GETPINFO_FULL;
+        else
+            snmp_log(LOG_ERR, "hr_swrun failed NT init\n");
+    } elif ((h = GetModuleHandle("KERNEL32.DLL")) != NULL) {
         myCreateToolhelp32Snapshot =
             (CREATESNAPSHOT) GetProcAddress(h, "CreateToolhelp32Snapshot");
         myProcess32First =
@@ -321,7 +311,7 @@ init_hr_swrun(void)
             && myProcess32Next)
 #if 0
             /*
-             * This doesn't work after all on Win98 SE 
+             * This doesn't work at all on Win98 SE
              */
             query = CW_GETPINFO_FULL;
 #else
@@ -698,13 +688,13 @@ var_hrswrun(struct variable * vp,
             *cp = '\0';
 #elif HAVE_KVM_GETPROCS
     #if defined(freebsd5) && __FreeBSD_version >= 500014
-        strcpy(string, proc_table[LowProcIndex].ki_comm);
+        strlcpy(string, proc_table[LowProcIndex].ki_comm, sizeof(string));
     #elif defined(dragonfly) && __DragonFly_version >= 190000
-        strcpy(string, proc_table[LowProcIndex].kp_comm);
+        strlcpy(string, proc_table[LowProcIndex].kp_comm, sizeof(string));
     #elif defined(openbsd5)
-        strcpy(string, proc_table[LowProcIndex].p_comm);
+        strlcpy(string, proc_table[LowProcIndex].p_comm, sizeof(string));
     #else
-        strcpy(string, proc_table[LowProcIndex].kp_proc.p_comm);
+        strlcpy(string, proc_table[LowProcIndex].kp_proc.p_comm, sizeof(string));
     #endif
 #elif defined(linux)
 	if( (cp=get_proc_name_from_status(pid,buf,sizeof(buf))) == NULL ) {
@@ -712,7 +702,7 @@ var_hrswrun(struct variable * vp,
             *var_len = strlen(string);
             return (u_char *) string;
         }
-        strcpy(string, cp);
+        strlcpy(string, cp, sizeof(string));
 #elif defined(cygwin)
         /* if (lowproc.process_state & (PID_ZOMBIE | PID_EXITED)) */
         if (lowproc.process_state & PID_EXITED || (lowproc.exitcode & ~0xffff))
@@ -721,7 +711,7 @@ var_hrswrun(struct variable * vp,
             cygwin_conv_to_posix_path(lowproc.progname, string);
             cp = strrchr(string, '/');
             if (cp)
-                strcpy(string, cp + 1);
+                strlcpy(string, cp + 1, sizeof(string));
         } else if (query == CW_GETPINFO_FULL) {
             DWORD           n = lowproc.dwProcessId & 0xffff;
             HANDLE          h =
@@ -739,7 +729,7 @@ var_hrswrun(struct variable * vp,
                                              sizeof string)) {
                     cp = strrchr(string, '\\');
                     if (cp)
-                        strcpy(string, cp + 1);
+                        strlcpy(string, cp + 1, sizeof(string));
                 } else
                     strcpy(string, "*** unknown");
                 CloseHandle(h);
@@ -795,7 +785,7 @@ var_hrswrun(struct variable * vp,
 #elif defined(solaris2)
 #ifdef _SLASH_PROC_METHOD_
         if (proc_buf)
-            strcpy(string, proc_buf->pr_psargs);
+            strlcpy(string, proc_buf->pr_psargs, sizeof(string));
         else
             sprintf(string, "<exited>");
         cp = strchr(string, ' ');
@@ -821,18 +811,18 @@ var_hrswrun(struct variable * vp,
             *cp = '\0';
 #elif HAVE_KVM_GETPROCS
     #if defined(freebsd5) && __FreeBSD_version >= 500014
-        strcpy(string, proc_table[LowProcIndex].ki_comm);
+        strlcpy(string, proc_table[LowProcIndex].ki_comm, sizeof(string));
     #elif defined(dragonfly) && __DragonFly_version >= 190000
-        strcpy(string, proc_table[LowProcIndex].kp_comm);
+        strlcpy(string, proc_table[LowProcIndex].kp_comm, sizeof(string));
     #elif defined(openbsd5)
-        strcpy(string, proc_table[LowProcIndex].p_comm);
+        strlcpy(string, proc_table[LowProcIndex].p_comm, sizeof(string));
     #else
-        strcpy(string, proc_table[LowProcIndex].kp_proc.p_comm);
+        strlcpy(string, proc_table[LowProcIndex].kp_proc.p_comm, sizeof(string));
     #endif
 #elif defined(linux)
         cp = get_proc_name_from_cmdline(pid,buf,sizeof(buf)-1);
         if (cp != NULL && *cp)    /* argv[0] '\0' argv[1] '\0' .... */
-            strcpy(string, cp);
+            strlcpy(string, cp, sizeof(string));
         else {
             /*
              * swapped out - no cmdline 
@@ -842,7 +832,7 @@ var_hrswrun(struct variable * vp,
 		*var_len = strlen(string);
 		return (u_char *) string;
 	    }
-            strcpy(string, cp);
+            strlcpy(string, cp, sizeof(string));
         }
 #elif defined(cygwin)
         /* if (lowproc.process_state & (PID_ZOMBIE | PID_EXITED)) */
@@ -900,7 +890,7 @@ var_hrswrun(struct variable * vp,
         if (proc_buf) {
             cp = strchr(proc_buf->pr_psargs, ' ');
             if (cp)
-                strcpy(string, cp + 1);
+                strlcpy(string, cp + 1, sizeof(string));
             else
                 string[0] = 0;
         } else
@@ -911,7 +901,7 @@ var_hrswrun(struct variable * vp,
             cp++;
         if (*cp == ' ')
             cp++;
-        strcpy(string, cp);
+        strlcpy(string, cp, sizeof(string));
 #endif
 #elif defined(aix4) || defined(aix5) || defined(aix6) || defined(aix7)
         cp = strchr(proc_table[LowProcIndex].pi_comm, ' ');
@@ -972,7 +962,7 @@ var_hrswrun(struct variable * vp,
         while (*cp)
             ++cp;
         ++cp;
-        strcpy(string, cp);
+        strlcpy(string, cp, sizeof(string));
 #elif defined(cygwin)
         string[0] = 0;
 #else
