@@ -10,15 +10,19 @@
 #include <net-snmp/agent/snmp_vars.h>
 #include "interface_private.h"
 
-netsnmp_feature_require(fd_event_manager)
-netsnmp_feature_require(delete_prefix_info)
-netsnmp_feature_require(create_prefix_info)
-netsnmp_feature_child_of(interface_arch_set_admin_status, interface_all)
+netsnmp_feature_require(fd_event_manager);
+netsnmp_feature_require(delete_prefix_info);
+netsnmp_feature_require(create_prefix_info);
+netsnmp_feature_child_of(interface_arch_set_admin_status, interface_all);
 
 #ifdef NETSNMP_FEATURE_REQUIRE_INTERFACE_ARCH_SET_ADMIN_STATUS
-netsnmp_feature_require(interface_ioctl_flags_set)
+netsnmp_feature_require(interface_ioctl_flags_set);
 #endif /* NETSNMP_FEATURE_REQUIRE_INTERFACE_ARCH_SET_ADMIN_STATUS */
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #ifdef HAVE_PCI_LOOKUP_NAME
 #include <pci/pci.h>
 #include <setjmp.h>
@@ -61,6 +65,7 @@ typedef __u8 u8;           /* ditto */
 
 #include "mibII/mibII_common.h"
 #include "if-mib/ifTable/ifTable_constants.h"
+#include "ip-mib/data_access/ipaddress_ioctl.h"
 
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
@@ -132,6 +137,41 @@ netsnmp_prefix_listen_info list_info;
 int netsnmp_prefix_listen(void);
 #endif
 
+#ifdef HAVE_PCI_LOOKUP_NAME
+static void init_libpci(void)
+{
+    struct stat stbuf;
+
+    /*
+     * When snmpd is run inside an OpenVZ container /proc/bus/pci is not
+     * available.
+     */
+    if (stat("/proc/vz", &stbuf) == 0)
+        return;
+
+    pci_access = pci_alloc();
+    if (!pci_access) {
+	snmp_log(LOG_ERR, "pcilib: pci_alloc failed\n");
+	return;
+    }
+
+    pci_access->error = netsnmp_pci_error;
+
+    do_longjmp = 1;
+    if (setjmp(err_buf)) {
+        pci_cleanup(pci_access);
+	snmp_log(LOG_ERR, "pcilib: pci_init failed\n");
+        pci_access = NULL;
+    }
+    else if (pci_access)
+	pci_init(pci_access);
+    do_longjmp = 0;
+}
+#else
+static void init_libpci(void)
+{
+}
+#endif
 
 void
 netsnmp_arch_interface_init(void)
@@ -171,25 +211,7 @@ netsnmp_arch_interface_init(void)
     netsnmp_prefix_listen();
 #endif
 
-#ifdef HAVE_PCI_LOOKUP_NAME
-    pci_access = pci_alloc();
-    if (!pci_access) {
-	snmp_log(LOG_ERR, "pcilib: pci_alloc failed\n");
-	return;
-    }
-
-    pci_access->error = netsnmp_pci_error;
-
-    do_longjmp = 1;
-    if (setjmp(err_buf)) {
-        pci_cleanup(pci_access);
-	snmp_log(LOG_ERR, "pcilib: pci_init failed\n");
-        pci_access = NULL;
-    }
-    else if (pci_access)
-	pci_init(pci_access);
-    do_longjmp = 0;
-#endif
+    init_libpci();
 }
 
 /*
@@ -463,7 +485,6 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
      *  [               OUT                               ]
      *   byte pkts errs drop fifo colls carrier compressed
      */
-#ifdef SCNuMAX
     uintmax_t       rec_pkt, rec_oct, rec_err, rec_drop, rec_mcast;
     uintmax_t       snd_pkt, snd_oct, snd_err, snd_drop, coll;
     const char     *scan_line_2_2 =
@@ -475,14 +496,6 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
         "%"   SCNuMAX " %"  SCNuMAX " %*" SCNuMAX " %*" SCNuMAX
         " %*" SCNuMAX " %"  SCNuMAX " %"  SCNuMAX " %*" SCNuMAX
         " %*" SCNuMAX " %"  SCNuMAX;
-#else
-    unsigned long   rec_pkt, rec_oct, rec_err, rec_drop, rec_mcast;
-    unsigned long   snd_pkt, snd_oct, snd_err, snd_drop, coll;
-    const char     *scan_line_2_2 =
-        "%lu %lu %lu %lu %*lu %*lu %*lu %lu %lu %lu %lu %lu %*lu %lu";
-    const char     *scan_line_2_0 =
-        "%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
-#endif
     static const char     *scan_line_to_use = NULL;
     int             scan_count;
 
@@ -522,10 +535,8 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
              */
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_SPEED;
-#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_BYTES;
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_PACKETS;
-#endif
         }
     } else {
         scan_count = sscanf(stats, scan_line_to_use,
@@ -561,13 +572,11 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
     entry->stats.imcast.low = rec_mcast & 0xffffffff;
     entry->stats.obytes.low = snd_oct & 0xffffffff;
     entry->stats.oucast.low = snd_pkt & 0xffffffff;
-#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
     entry->stats.ibytes.high = rec_oct >> 32;
     entry->stats.iall.high = rec_pkt >> 32;
     entry->stats.imcast.high = rec_mcast >> 32;
     entry->stats.obytes.high = snd_oct >> 32;
     entry->stats.oucast.high = snd_pkt >> 32;
-#endif
     entry->stats.ierrors   = rec_err;
     entry->stats.idiscards = rec_drop;
     entry->stats.oerrors   = snd_err;
@@ -603,6 +612,8 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
     netsnmp_interface_entry *entry = NULL;
     static char     scan_expected = 0;
     int             fd;
+    int             interfaces = 0;
+    struct ifconf   ifc;
 #ifdef NETSNMP_ENABLE_IPV6
     netsnmp_container *addr_container;
 #endif
@@ -648,8 +659,8 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
      * to detect the position of individual fields directly,
      * but I suspect this is probably more trouble than it's worth.
      */
-    fgets(line, sizeof(line), devin);
-    fgets(line, sizeof(line), devin);
+    NETSNMP_IGNORE_RESULT(fgets(line, sizeof(line), devin));
+    NETSNMP_IGNORE_RESULT(fgets(line, sizeof(line), devin));
 
     if( 0 == scan_expected ) {
         if (strstr(line, "compressed")) {
@@ -663,6 +674,16 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         }
     }
 
+    interfaces = netsnmp_access_ipaddress_ioctl_get_interface_count(fd, &ifc);
+    if (interfaces < 0) {
+        snmp_log(LOG_ERR,"get interface count failed\n");
+        fclose(devin);
+        close(fd);
+        return -2;
+    }
+    netsnmp_assert(NULL != ifc.ifc_buf);
+
+
     /*
      * The rest of the file provides the statistics for each interface.
      * Read in each line in turn, isolate the interface name
@@ -700,8 +721,12 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
          */
         *stats++ = 0; /* null terminate name */
 
-        if_index = netsnmp_arch_interface_index_find(ifstart);
+	if (!netsnmp_access_interface_include(ifstart))
+		continue;
 
+	if (netsnmp_access_interface_max_reached(ifstart))
+		/* we may need to stop tracking ifaces if a max was set */
+		continue;
         /*
          * set address type flags.
          * the only way I know of to check an interface for
@@ -709,9 +734,10 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
          * knows a better way, put it here!
          */
 #ifdef NETSNMP_ENABLE_IPV6
+        if_index = netsnmp_arch_interface_index_find(ifstart);
         _arch_interface_has_ipv6(if_index, &flags, addr_container);
 #endif
-        netsnmp_access_interface_ioctl_has_ipv4(fd, ifstart, 0, &flags);
+        netsnmp_access_interface_ioctl_has_ipv4(fd, ifstart, 0, &flags, &ifc);
 
         /*
          * do we only want one address type?
@@ -735,6 +761,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
                                                     NETSNMP_ACCESS_INTERFACE_FREE_NOFLAGS);
             fclose(devin);
             close(fd);
+            free(ifc.ifc_buf);
             return -3;
         }
         entry->ns_flags = flags; /* initial flags; we'll set more later */
@@ -902,6 +929,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
 #endif
     fclose(devin);
     close(fd);
+    free(ifc.ifc_buf);
     return 0;
 }
 
@@ -926,43 +954,29 @@ netsnmp_arch_set_admin_status(netsnmp_interface_entry * entry,
 
 #ifdef HAVE_LINUX_ETHTOOL_H
 /**
- * Determines network interface speed from ETHTOOL_GSET
+ * Determines network interface speed from ETHTOOL_GLINKSETTINGS
+ * In case of failure revert to obsolete ETHTOOL_GSET
  */
 unsigned long long
 netsnmp_linux_interface_get_if_speed(int fd, const char *name,
             unsigned long long defaultspeed)
 {
     int ret;
-    struct ifreq ifr;
-    struct ethtool_cmd edata;
-    uint16_t speed_hi;
-    uint32_t speed;
+    struct netsnmp_linux_link_settings nlls;
+    uint32_t speed = -1;
 
-    memset(&ifr, 0, sizeof(ifr));
-    memset(&edata, 0, sizeof(edata));
-    edata.cmd = ETHTOOL_GSET;
-    
-    strlcpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
-    ifr.ifr_data = (char *) &edata;
-    
-    ret = ioctl(fd, SIOCETHTOOL, &ifr);
-    if (ret == -1 || edata.speed == 0) {
+    ret = netsnmp_get_link_settings(&nlls, fd, name);
+    if (ret < 0) {
         DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s failed (%d / %d)\n",
-                    ifr.ifr_name, ret, edata.speed));
-        return netsnmp_linux_interface_get_if_speed_mii(fd,name,defaultspeed);
+                    name, ret, speed));
+        return netsnmp_linux_interface_get_if_speed_mii(fd, name, defaultspeed);
     }
-
-#ifdef HAVE_STRUCT_ETHTOOL_CMD_SPEED_HI
-    speed_hi = edata.speed_hi;
-#else
-    speed_hi = 0;
-#endif
-    speed = speed_hi << 16 | edata.speed;
+    speed = nlls.speed;
     if (speed == 0xffff || speed == 0xffffffffUL /*SPEED_UNKNOWN*/)
         speed = defaultspeed;
     /* return in bps */
-    DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s speed = %#x -> %d\n",
-                ifr.ifr_name, speed_hi << 16 | edata.speed, speed));
+    DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s speed = %#x = %d\n",
+                name, speed, speed));
     return speed * 1000LL * 1000LL;
 }
 #endif
