@@ -31,7 +31,7 @@
 #include <net/if_types.h>
 #include <net/if_media.h>
 
-netsnmp_feature_child_of(interface_arch_set_admin_status, interface_all)
+netsnmp_feature_child_of(interface_arch_set_admin_status, interface_all);
 
 /*
  * account for minor differences between FreeBSD and OpenBSD.
@@ -136,7 +136,7 @@ netsnmp_sysctl_ifmedia_to_speed(int media, u_int *speed,
                     *speed = (u_int) -1; /* 4294967295; */
                     *speed_high = 10000; break;
 #endif
-#if defined(__FreeBSD__)
+#if defined(freebsd3)
 #ifdef IFM_10G_LR
                 case IFM_10G_LR:
 #ifdef IFM_10G_SR
@@ -248,7 +248,7 @@ netsnmp_sysctl_ifmedia_to_speed(int media, u_int *speed,
                     *speed = (u_int) -1; /* 4294967295; */
                     *speed_high = 100000; break;
 #endif /* IFM_100G_CR4 */
-#endif /* __FreeBSD__ */
+#endif /* freebsd3 */
             }
             break;
         case IFM_IEEE80211:
@@ -531,8 +531,11 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
                 }
             }
             adl = (struct sockaddr_dl *) a;
-            if_name = (char *) adl->sdl_data;
-            if_name[adl->sdl_nlen] = '\0';
+            if_name = malloc(adl->sdl_nlen + 1);
+            if (if_name) {
+                memcpy(if_name, adl->sdl_data, adl->sdl_nlen);
+                if_name[adl->sdl_nlen] = '\0';
+            }
         }
         if (!(ifp->ifm_addrs & RTA_IFP) || if_name == NULL) {
             snmp_log(LOG_ERR, "ifm_index %u: no interface name in message, "
@@ -541,6 +544,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         }
 
         entry = netsnmp_access_interface_entry_create(if_name, ifp->ifm_index);
+        free(if_name);
         if(NULL == entry) {
             netsnmp_access_interface_container_free(container,
                                                     NETSNMP_ACCESS_INTERFACE_FREE_NOFLAGS);
