@@ -8,10 +8,13 @@
 
 config_require(mibII/tcpTable)
 
-config_arch_require(solaris2,        kernel_sunos5)
-config_arch_require(linux,     mibII/kernel_linux)
-config_arch_require(netbsd,    mibII/kernel_netbsd)
-config_arch_require(netbsdelf, mibII/kernel_netbsd)
+#ifdef solaris2
+config_require(kernel_sunos5)
+#elif defined(linux)
+config_require(mibII/kernel_linux)
+#elif defined(netbsd) || defined(netbsdelf)
+config_require(mibII/kernel_netbsd)
+#endif
 
 extern void     init_tcp(void);
 extern Netsnmp_Node_Handler tcp_handler;
