@@ -23,13 +23,13 @@ config_require(tcp-mib/data_access/tcpConn_linux)
 config_require(util_funcs/get_pid_from_inode)
 #elif defined( solaris2 )
 config_require(tcp-mib/data_access/tcpConn_solaris2)
-#elif defined(freebsd4) || defined(dragonfly)
+#elif defined(freebsd4) || defined(dragonfly) || defined(darwin)
 config_require(tcp-mib/data_access/tcpConn_freebsd4)
 #elif defined(openbsd4)
 config_require(tcp-mib/data_access/tcpConn_openbsd)
 #elif defined(netbsd1)
 config_require(tcp-mib/data_access/tcpConn_netbsd)
 #else
-#   define NETSNMP_TCPCONN_COMMON_ONLY
+config_require(tcp-mib/data_access/tcpConn_unsup)
 #endif
 
