@@ -6,6 +6,11 @@
  * Copyright Copyright 2003 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -80,9 +85,23 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#ifdef HAVE_LINUX_ETHTOOL_H
+#include <sys/ioctl.h>
+#include <linux/if.h>
+#include <linux/sockios.h>
+#ifdef HAVE_LINUX_ETHTOOL_NEEDS_U64
+#include <linux/types.h>
+typedef __u64 u64;         /* hack, so we may include kernel's ethtool.h */
+typedef __u32 u32;         /* ditto */
+typedef __u16 u16;         /* ditto */
+typedef __u8 u8;           /* ditto */
+#endif
+#include <linux/ethtool.h>
+#endif /* HAVE_LINUX_ETHTOOL_H */
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/library/snmp_logging.h>
+#include <net-snmp/agent/netsnmp_close_fds.h>
 
 #include "struct.h"
 #include "util_funcs.h"
@@ -97,31 +116,29 @@
 #define setPerrorstatus(x) snmp_log_perror(x)
 #endif
 
-netsnmp_feature_child_of(util_funcs, libnetsnmpmibs)
+netsnmp_feature_child_of(util_funcs, libnetsnmpmibs);
 
-netsnmp_feature_child_of(shell_command, util_funcs)
-netsnmp_feature_child_of(get_exten_instance, util_funcs)
-netsnmp_feature_child_of(clear_cache, util_funcs)
-netsnmp_feature_child_of(find_field, util_funcs)
-netsnmp_feature_child_of(parse_miboid, util_funcs)
-netsnmp_feature_child_of(string_append_int, util_funcs)
-netsnmp_feature_child_of(internal_mib_table, util_funcs)
+netsnmp_feature_child_of(shell_command, util_funcs);
+netsnmp_feature_child_of(get_exten_instance, util_funcs);
+netsnmp_feature_child_of(clear_cache, util_funcs);
+netsnmp_feature_child_of(find_field, util_funcs);
+netsnmp_feature_child_of(parse_miboid, util_funcs);
+netsnmp_feature_child_of(string_append_int, util_funcs);
+netsnmp_feature_child_of(internal_mib_table, util_funcs);
 
 #if defined(HAVE_LINUX_RTNETLINK_H)
-netsnmp_feature_child_of(prefix_info_all, util_funcs)
-netsnmp_feature_child_of(prefix_info, prefix_info_all)
-netsnmp_feature_child_of(update_prefix_info, prefix_info_all)
-netsnmp_feature_child_of(delete_prefix_info, prefix_info_all)
-netsnmp_feature_child_of(find_prefix_info, prefix_info_all)
-netsnmp_feature_child_of(create_prefix_info, prefix_info_all)
+netsnmp_feature_child_of(prefix_info_all, util_funcs);
+netsnmp_feature_child_of(prefix_info, prefix_info_all);
+netsnmp_feature_child_of(update_prefix_info, prefix_info_all);
+netsnmp_feature_child_of(delete_prefix_info, prefix_info_all);
+netsnmp_feature_child_of(find_prefix_info, prefix_info_all);
+netsnmp_feature_child_of(create_prefix_info, prefix_info_all);
 #endif /* HAVE_LINUX_RTNETLINK_H */
 
-#ifdef NETSNMP_EXCACHETIME
+#if defined(NETSNMP_EXCACHETIME) && defined(USING_UTILITIES_EXECUTE_MODULE) && defined(HAVE_EXECV)
 static long     cachetime;
 #endif
 
-extern int      numprocs, numextens;
-
 /** deprecated, use netsnmp_mktemp instead */
 const char *
 make_tempfile(void)
@@ -135,7 +152,7 @@ shell_command(struct extensible *ex)
 {
 #if HAVE_SYSTEM
     const char     *ofname;
-    char            shellline[STRMAX];
+    char           *shellline = NULL;
     FILE           *shellout;
 
     ofname = make_tempfile();
@@ -145,10 +162,11 @@ shell_command(struct extensible *ex)
         return ex->result;
     }
 
-    snprintf(shellline, sizeof(shellline), "%s > %s", ex->command, ofname);
-    shellline[ sizeof(shellline)-1 ] = 0;
-    ex->result = system(shellline);
-    ex->result = WEXITSTATUS(ex->result);
+    if (asprintf(&shellline, "%s > %s", ex->command, ofname) >= 0) {
+        ex->result = system(shellline);
+        ex->result = WEXITSTATUS(ex->result);
+        free(shellline);
+    }
     shellout = fopen(ofname, "r");
     if (shellout != NULL) {
         if (fgets(ex->output, sizeof(ex->output), shellout) == NULL) {
@@ -258,7 +276,7 @@ get_exec_output(struct extensible *ex)
     curtime = time(NULL);
     if (curtime > (cachetime + NETSNMP_EXCACHETIME) ||
         strcmp(ex->command, lastcmd) != 0) {
-        strcpy(lastcmd, ex->command);
+        strlcpy(lastcmd, ex->command, sizeof(lastcmd));
         cachetime = curtime;
 #endif
 
@@ -278,7 +296,7 @@ get_exec_output(struct extensible *ex)
                 return -1;
         }
         if (cachebytes > 0)
-            write(cfd, (void *) cache, cachebytes);
+            NETSNMP_IGNORE_RESULT(write(cfd, cache, cachebytes));
         close(cfd);
 #ifdef NETSNMP_EXCACHETIME
         lastresult = ex->result;
@@ -293,9 +311,8 @@ get_exec_output(struct extensible *ex)
         return -1;
     }
     return (cfd);
-#else                           /* !HAVE_EXECV */
-#if defined(WIN32) && !defined(HAVE_EXECV)
-/* MSVC and MinGW.  Cygwin already works as it has execv and fork */
+#elif defined(HAVE__GET_OSFHANDLE) && defined(HAVE__OPEN_OSFHANDLE)
+    /* MSVC and MinGW32. Cygwin has execv() and fork(). */
     int         fd;   
     
     /* Reference:  MS tech note: 190351 */
@@ -346,7 +363,7 @@ get_exec_output(struct extensible *ex)
     }
     
     /* Associates a C run-time file descriptor with an existing operating-system file handle. */
-    fd = _open_osfhandle((long) hOutputRead, 0);
+    fd = _open_osfhandle((intptr_t) hOutputRead, 0);
     
     /* Set up STARTUPINFO for CreateProcess with the handles and have it hide the window
      * for the new process. */
@@ -388,70 +405,69 @@ get_exec_output(struct extensible *ex)
       return -1;
     }
     return fd;
-#endif                          /* WIN32 */
-#endif
+#endif       /* HAVE_EXECV */
 #endif /* !defined(USING_UTILITIES_EXECUTE_MODULE) */
     return -1;
 }
 int
 get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
 {
-/* 	Alexander Prömel, alexander@proemel.de 08/24/2006
-	The following code, is tested on picotux rev. 1.01.
-	I think, it will be better to put the named pipes, into /var/run or make it selectable via CONFIG file.
-	If the pipe file already exist, the creation will fail.
-	I put the pipes into /flash, the pipepath has to change in ucd-snmp/pass_persist.c too, if you change it here.
-*/
-#if HAVE_EXECV
-#ifdef __uClinux__ /* HAVE uClinux */
-	int in,out;
-	char fifo_in_path[256];
-	char fifo_out_path[256];
-	pid_t tpid;
-        
-    if ((tpid = vfork()) == 0) { /*temp child*/
+#if defined(HAVE_EXECV) && defined(__uClinux__)
+    int in, out;
+    char fifo_in_path[256];
+    char fifo_out_path[256];
+    pid_t tpid;
+
+    /*
+     * Alexander Prömel, alexander@proemel.de 08/24/2006. The following code,
+     * is tested on picotux rev. 1.01.  I think, it will be better to put the
+     * named pipes, into /var/run or make it selectable via CONFIG file. If
+     * the pipe file already exist, the creation will fail. I put the pipes
+     * into /flash, the pipepath has to change in ucd-snmp/pass_persist.c
+     * too, if you change it here.
+     */
+    if ((tpid = vfork()) == 0) {
+        /* child */
         execve(cmd, NULL,NULL);
         perror(cmd);
         exit(1);
-    } else { 
-		if(tpid > 0) {
-			/*initialize workspace*/
-			snprintf(fifo_in_path, 256, "/flash/cp_%d", tpid);
-			snprintf(fifo_out_path, 256, "/flash/pc_%d", tpid);
+    } else if (tpid > 0) {
+        /* parent */
+        snprintf(fifo_in_path, sizeof(fifo_in_path), "/flash/cp_%d", tpid);
+        snprintf(fifo_out_path, sizeof(fifo_out_path), "/flash/pc_%d", tpid);
 
-			in = mkfifo(fifo_in_path, S_IRWXU);	/*Create Input Pipe, 700*/
-			if ( in ) {
-				perror("parent: inpipe");
-				exit(0);
-			}
-			out = mkfifo(fifo_out_path, S_IRWXU);	/*Create Output Pipe, 700*/
-			if ( out ) {
-				perror("parent: outpipe");
-				exit(0);
-			}
-						
-			in = open(fifo_in_path,O_RDONLY);	/*open the Input Pipe read Only*/
-			if(in < 0) {
-				perror("parent: input");
-				exit(0);
-			}
-			out = open(fifo_out_path,O_WRONLY); 	/*open the Output Pipe write Only*/
-			if(out < 0) {
-				perror("parent: output");
-				exit(0);
-			}
-			
-			*fdIn = in;	/*read*/
-			*fdOut = out;	/*write*/
-			*pid = tpid;	
-			return (1);     /* We are returning 0 for error... */
-		} else { /*pid < 0*/
-			setPerrorstatus("vfork");
-			return 0;
-		}
+        in = mkfifo(fifo_in_path, S_IRWXU);	/*Create Input Pipe, 700*/
+        if (in) {
+            perror("parent: inpipe");
+            return 0;
+        }
+        out = mkfifo(fifo_out_path, S_IRWXU);	/*Create Output Pipe, 700*/
+        if (out) {
+            perror("parent: outpipe");
+            return 0;
+        }
 
+        in = open(fifo_in_path,O_RDONLY);	/*open the Input Pipe read Only*/
+        if (in < 0) {
+            perror("parent: input");
+            return 0;
+        }
+        out = open(fifo_out_path,O_WRONLY); 	/*open the Output Pipe write Only*/
+        if (out < 0) {
+            perror("parent: output");
+            return 0;
+        }
+
+        *fdIn = in;	/*read*/
+        *fdOut = out;	/*write*/
+        *pid = tpid;
+        return 1;
+    } else {
+        /* pid < 0 */
+        setPerrorstatus("vfork");
+        return 0;
     }
-#else /*HAVE x86*/
+#elif defined(HAVE_EXECV)
     int             fd[2][2], i, cnt;
     char            ctmp[STRMAX], *cptr1, *cptr2, argvs[STRMAX], **argv,
         **aptr;
@@ -463,16 +479,20 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
         return 0;
     }
     if ((*pid = fork()) == 0) { /* First handle for the child */
+        close(fd[0][1]);
+        close(fd[1][0]);
         close(0);
         if (dup(fd[0][0]) != 0) {
             setPerrorstatus("dup 0");
             return 0;
         }
+        close(fd[0][0]);
         close(1);
         if (dup(fd[1][1]) != 1) {
             setPerrorstatus("dup 1");
             return 0;
         }
+        close(fd[1][1]);
 
         /*
          * write standard output and standard error to pipe. 
@@ -480,9 +500,8 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
         /*
          * close all non-standard open file descriptors 
          */
-        for (cnt = getdtablesize() - 1; cnt >= 2; --cnt)
-            (void) close(cnt);
-        (void) dup(1);          /* stderr */
+        netsnmp_close_fds(1);
+        NETSNMP_IGNORE_RESULT(dup(1));  /* stderr */
 
         for (cnt = 1, cptr1 = cmd, cptr2 = argvs; *cptr1 != 0;
              cptr2++, cptr1++) {
@@ -528,10 +547,8 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
         *fdOut = fd[0][1];
         return (1);             /* We are returning 0 for error... */
     }
-#endif				/* uClinux or x86 */
-#endif                          /* !HAVE_EXECV */
-#if defined(WIN32) && !defined (mingw32) && !defined(HAVE_EXECV)
-/* MSVC (MinGW not working but should use this code).  Cygwin already works as it has execv and fork */
+#elif defined(HAVE__GET_OSFHANDLE) && defined(HAVE__OPEN_OSFHANDLE)
+    /* MSVC and MinGW32. Cygwin has execv() and fork(). */
     /* Reference:  MS tech note: 190351 */
     HANDLE hInputWriteTmp, hInputRead, hInputWrite = NULL;
     HANDLE hOutputReadTmp, hOutputRead, hOutputWrite = NULL;
@@ -548,12 +565,13 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
     /* Child temporary output pipe with Inheritance on (sa.bInheritHandle is true) */    
     if (!CreatePipe(&hOutputReadTmp,&hOutputWrite,&sa,0)) {
       DEBUGMSGTL(("util_funcs", "get_exec_pipes CreatePipe ChildOut: %d\n",
-            GetLastError()));
+                  (unsigned int)GetLastError()));
       return 0;
     }
     /* Child temporary input pipe with Inheritance on (sa.bInheritHandle is true) */    
     if (!CreatePipe(&hInputRead,&hInputWriteTmp,&sa,0)) {
-      DEBUGMSGTL(("util_funcs", "get_exec_pipes CreatePipe ChildIn: %d\n", GetLastError()));
+        DEBUGMSGTL(("util_funcs", "get_exec_pipes CreatePipe ChildIn: %d\n",
+                    (unsigned int)GetLastError()));
       return 0;
     }
     
@@ -561,46 +579,55 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
      * its stdout handles. */
     if (!DuplicateHandle(GetCurrentProcess(),hOutputWrite, GetCurrentProcess(),
           &hErrorWrite,0, TRUE,DUPLICATE_SAME_ACCESS)) {
-      DEBUGMSGTL(("util_funcs", "get_exec_pipes DuplicateHandle: %d\n", GetLastError()));
-      return 0;
+        DEBUGMSGTL(("util_funcs", "get_exec_pipes DuplicateHandle: %d\n",
+                    (unsigned int)GetLastError()));
+        return 0;
     }
 
     /* Create new copies of the input and output handles but set bInheritHandle to 
      * FALSE so the new handle can not be inherited.  Otherwise the handles can not
      * be closed.  */
-    if (!DuplicateHandle(GetCurrentProcess(), hOutputReadTmp, GetCurrentProcess(),
-          &hOutputRead, 0, FALSE, DUPLICATE_SAME_ACCESS)) {
-      DEBUGMSGTL(("util_funcs", "get_exec_pipes DupliateHandle ChildOut: %d\n", GetLastError()));
-      CloseHandle(hErrorWrite);
-      return 0;
+    if (!DuplicateHandle(GetCurrentProcess(), hOutputReadTmp,
+                         GetCurrentProcess(), &hOutputRead, 0, FALSE,
+                         DUPLICATE_SAME_ACCESS)) {
+        DEBUGMSGTL(("util_funcs",
+                    "get_exec_pipes DupliateHandle ChildOut: %d\n",
+                    (unsigned int)GetLastError()));
+        CloseHandle(hErrorWrite);
+        return 0;
     }   
     if (!DuplicateHandle(GetCurrentProcess(),hInputWriteTmp,
           GetCurrentProcess(), &hInputWrite, 0, FALSE, DUPLICATE_SAME_ACCESS)) {
-      DEBUGMSGTL(("util_funcs","get_exec_pipes DupliateHandle ChildIn: %d\n", GetLastError()));
-      CloseHandle(hErrorWrite);
-      CloseHandle(hOutputRead);
-      return 0;
+        DEBUGMSGTL(("util_funcs","get_exec_pipes DupliateHandle ChildIn: %d\n",
+                    (unsigned int)GetLastError()));
+        CloseHandle(hErrorWrite);
+        CloseHandle(hOutputRead);
+        return 0;
     }
 
     /* Close the temporary output and input handles */
     if (!CloseHandle(hOutputReadTmp)) {
-      DEBUGMSGTL(("util_funcs", "get_exec_pipes CloseHandle (hOutputReadTmp): %d\n", GetLastError()));
-      CloseHandle(hErrorWrite);
-      CloseHandle(hOutputRead);
-      CloseHandle(hInputWrite);
-      return 0;
+        DEBUGMSGTL(("util_funcs",
+                    "get_exec_pipes CloseHandle (hOutputReadTmp): %d\n",
+                    (unsigned int)GetLastError()));
+        CloseHandle(hErrorWrite);
+        CloseHandle(hOutputRead);
+        CloseHandle(hInputWrite);
+        return 0;
     }
     if (!CloseHandle(hInputWriteTmp)) {
-      DEBUGMSGTL(("util_funcs", "get_exec_pipes CloseHandle (hInputWriteTmp): %d\n", GetLastError()));
-      CloseHandle(hErrorWrite);
-      CloseHandle(hOutputRead);
-      CloseHandle(hInputWrite);
-      return 0;
+        DEBUGMSGTL(("util_funcs",
+                    "get_exec_pipes CloseHandle (hInputWriteTmp): %d\n",
+                    (unsigned int)GetLastError()));
+        CloseHandle(hErrorWrite);
+        CloseHandle(hOutputRead);
+        CloseHandle(hInputWrite);
+        return 0;
     }
     
     /* Associates a C run-time file descriptor with an existing operating-system file handle. */
-    *fdIn = _open_osfhandle((long) hOutputRead, 0);
-    *fdOut = _open_osfhandle((long) hInputWrite, 0);
+    *fdIn = _open_osfhandle((intptr_t) hOutputRead, 0);
+    *fdOut = _open_osfhandle((intptr_t) hInputWrite, 0);
     
     /* Set up STARTUPINFO for CreateProcess with the handles and have it hide the window
      * for the new process. */
@@ -616,14 +643,15 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
      * pass_persist    .1.3.6.1.4.1.2021.255  c:/perl/bin/perl c:/temp/pass_persisttest
     */
     if (!CreateProcess(NULL, cmd, NULL, NULL, TRUE, CREATE_NEW_CONSOLE, NULL, NULL, &si, &pi)) {
-      DEBUGMSGTL(("util_funcs","get_exec_pipes CreateProcess:'%s' %d\n",cmd, GetLastError()));
-      CloseHandle(hErrorWrite);
-      CloseHandle(hOutputRead);
-      CloseHandle(hInputWrite);
-      return 0;
+        DEBUGMSGTL(("util_funcs","get_exec_pipes CreateProcess:'%s' %d\n", cmd,
+                    (unsigned int)GetLastError()));
+        CloseHandle(hErrorWrite);
+        CloseHandle(hOutputRead);
+        CloseHandle(hInputWrite);
+        return 0;
     }
     
-    DEBUGMSGTL(("util_funcs","child hProcess (stored in pid): %d\n",(int)pi.hProcess));
+    DEBUGMSGTL(("util_funcs","child hProcess (stored in pid): %p\n", pi.hProcess));
     DEBUGMSGTL(("util_funcs","child dwProcessId (task manager): %d\n",(int)pi.dwProcessId));
 
     /* Set global child process handle */
@@ -631,7 +659,9 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
 
     /* Cleanup */
     if (!CloseHandle(pi.hThread))
-      DEBUGMSGTL(("util_funcs","get_exec_pipes CloseHandle pi.hThread: %d\n",cmd));
+      DEBUGMSGTL(("util_funcs",
+		  "get_exec_pipes(%s) CloseHandle pi.hThread: %d\n",
+		  cmd, (unsigned int)GetLastError()));
 
    /* Close pipe handles to make sure that no handles to the write end of the
      * output pipe are maintained in this process or else the pipe will
@@ -640,15 +670,21 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, netsnmp_pid_t *pid)
      */
 
     if (!CloseHandle(hOutputWrite)){
-      DEBUGMSGTL(("util_funcs","get_exec_pipes CloseHandle hOutputWrite: %d\n",cmd, GetLastError()));
+      DEBUGMSGTL(("util_funcs",
+		  "get_exec_pipes(%s) CloseHandle hOutputWrite: %d\n",
+                  cmd, (unsigned int)GetLastError()));
       return 0;
     }
     if (!CloseHandle(hInputRead)) {
-      DEBUGMSGTL(("util_funcs","get_exec_pipes CloseHandle hInputRead: %d\n",cmd, GetLastError()));
+      DEBUGMSGTL(("util_funcs",
+		  "get_exec_pipes(%s) CloseHandle hInputRead: %d\n",
+                  cmd, (unsigned int)GetLastError()));
       return 0;
     }
     if (!CloseHandle(hErrorWrite)) {
-      DEBUGMSGTL(("util_funcs","get_exec_pipes CloseHandle hErrorWrite: %d\n",cmd, GetLastError()));
+      DEBUGMSGTL(("util_funcs",
+		  "get_exec_pipes(%s) CloseHandle hErrorWrite: %d\n",
+                  cmd, (unsigned int)GetLastError()));
       return 0;
     }
     return 1;
@@ -664,19 +700,19 @@ clear_cache(int action,
             size_t var_val_len,
             u_char * statP, oid * name, size_t name_len)
 {
-
-    long            tmp = 0;
-
     if (var_val_type != ASN_INTEGER) {
         snmp_log(LOG_NOTICE, "Wrong type != int\n");
         return SNMP_ERR_WRONGTYPE;
     }
-    tmp = *((long *) var_val);
-    if (tmp == 1 && action == COMMIT) {
-#ifdef NETSNMP_EXCACHETIME
-        cachetime = 0;          /* reset the cache next read */
-#endif
+#if defined(NETSNMP_EXCACHETIME) && defined(USING_UTILITIES_EXECUTE_MODULE) && defined(HAVE_EXECV)
+    else {
+        long            tmp = 0;
+        tmp = *((long *) var_val);
+        if (tmp == 1 && action == COMMIT) {
+            cachetime = 0;          /* reset the cache next read */
+        }
     }
+#endif
     return SNMP_ERR_NOERROR;
 }
 #endif /* NETSNMP_FEATURE_REMOVE_CLEAR_CACHE */
@@ -1111,7 +1147,7 @@ int net_snmp_delete_prefix_info(prefix_cbx **head,
     for (temp_node = *head, prev_node = NULL; temp_node;
          prev_node = temp_node, temp_node = temp_node->next_info) {
 
-         if (temp_node->in6p && strcmp(temp_node->in6p, address) == 0) {
+         if (strcmp(temp_node->in6p, address) == 0) {
             if (prev_node)
                 prev_node->next_info = temp_node->next_info;
             else
@@ -1127,3 +1163,77 @@ int net_snmp_delete_prefix_info(prefix_cbx **head,
 
 #endif /* HAVE_LINUX_RTNETLINK_H */
          
+/**
+ * netsnmp_get_link_settings
+ * @name
+ *
+ * @returns  0 success
+ * @returns -1 Both ETHTOOL_GLINKSETTINGS and ETHTOOL_GSET failed
+ * @returns -2 HAVE_LINUX_ETHTOOL_H is not defined
+ */
+int netsnmp_get_link_settings(struct netsnmp_linux_link_settings *nlls,
+                              int fd, const char *name)
+{
+    int err = -2;
+
+#ifdef HAVE_LINUX_ETHTOOL_H
+    struct ifreq ifr;
+
+    err = -1;
+    memset(&ifr, 0, sizeof(ifr));
+    strlcpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
+
+#ifdef ETHTOOL_GLINKSETTINGS
+    {
+        /*
+         * For Linux kernel v5.2 __ETHTOOL_LINK_MODE_MASK_NBITS == 67 or
+         * 3 32-bit words. Increase the 'nwords' constant if necessary.
+         */
+        enum { mask_nwords = 4 };
+        struct {
+            struct ethtool_link_settings elinkset;
+            uint32_t masks[3 * mask_nwords];
+        } data;
+
+        memset(&data, 0, sizeof(data));
+        data.elinkset.cmd = ETHTOOL_GLINKSETTINGS;
+        ifr.ifr_data = &data.elinkset;
+        err = ioctl(fd, SIOCETHTOOL, &ifr, name);
+        /*
+         * See also the struct ethtool_link_settings documentation in
+         * Linux kernel header file include/uapi/linux/ethtool.h.
+         */
+        if (data.elinkset.link_mode_masks_nwords < 0 &&
+            -data.elinkset.link_mode_masks_nwords <= mask_nwords) {
+            data.elinkset.link_mode_masks_nwords =
+                -data.elinkset.link_mode_masks_nwords;
+            err = ioctl(fd, SIOCETHTOOL, &ifr);
+        }
+        if (err >= 0) {
+            nlls->duplex = data.elinkset.duplex;
+            nlls->speed = data.elinkset.speed;
+        }
+    }
+#endif /* ETHTOOL_GLINKSETTINGS */
+
+    if (err < 0) {
+        struct ethtool_cmd edata;
+
+        memset(&edata, 0, sizeof(edata));
+        edata.cmd = ETHTOOL_GSET;
+        ifr.ifr_data = (char *)&edata;
+        err = ioctl(fd, SIOCETHTOOL, &ifr, name);
+        if (err >= 0) {
+            nlls->duplex = edata.duplex;
+            nlls->speed = edata.speed;
+#ifdef HAVE_STRUCT_ETHTOOL_CMD_SPEED_HI
+            nlls->speed |= edata.speed_hi << 16;
+#endif
+        } else {
+            err = -1;
+        }
+    }
+#endif /* HAVE_LINUX_ETHTOOL_H */
+
+    return err;
+}
