@@ -103,7 +103,7 @@
 #include "util_funcs.h"
 #include "winExtDLL.h"
 
-netsnmp_feature_require(oid_is_subtree)
+netsnmp_feature_require(oid_is_subtree);
 
 
 #define MAX_VALUE_NAME          16383
@@ -316,9 +316,8 @@ init_winExtDLL(void)
 
     GetVersionEx(&s_versioninfo);
 
-    IsWow64Process =
-      (pfIsWow64Process)GetProcAddress(GetModuleHandle("kernel32"),
-                                       "IsWow64Process");
+    IsWow64Process = (pfIsWow64Process)(uintptr_t)
+	GetProcAddress(GetModuleHandle("kernel32"), "IsWow64Process");
     if (IsWow64Process)
         (*IsWow64Process)(GetCurrentProcess(), &is_wow64_process);
 
@@ -378,19 +377,21 @@ init_winExtDLL(void)
         /*
          * Store DLL name and functions in s_extension_dll_info array. 
          */
-        ext_dll_info->pfSnmpExtensionInit = (PFNSNMPEXTENSIONINIT)
+        ext_dll_info->pfSnmpExtensionInit = (PFNSNMPEXTENSIONINIT)(uintptr_t)
             GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionInit");
-        ext_dll_info->pfSnmpExtensionInitEx = (PFNSNMPEXTENSIONINITEX)
+        ext_dll_info->pfSnmpExtensionInitEx =
+	    (PFNSNMPEXTENSIONINITEX)(uintptr_t)
             GetProcAddress(ext_dll_info->dll_handle,
                            "SnmpExtensionInitEx");
         ext_dll_info->pfSnmpExtensionClose = (PFNSNMPEXTENSIONCLOSE)
             GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionClose");
-        ext_dll_info->pfSnmpExtensionQuery = (PFNSNMPEXTENSIONQUERY)
+        ext_dll_info->pfSnmpExtensionQuery = (PFNSNMPEXTENSIONQUERY)(uintptr_t)
             GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionQuery");
-        ext_dll_info->pfSnmpExtensionQueryEx = (PFNSNMPEXTENSIONQUERYEX)
+        ext_dll_info->pfSnmpExtensionQueryEx =
+	    (PFNSNMPEXTENSIONQUERYEX)(uintptr_t)
             GetProcAddress(ext_dll_info->dll_handle,
                            "SnmpExtensionQueryEx");
-        ext_dll_info->pfSnmpExtensionTrap = (PFNSNMPEXTENSIONTRAP)
+        ext_dll_info->pfSnmpExtensionTrap = (PFNSNMPEXTENSIONTRAP)(uintptr_t)
             GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionTrap");
 
 
@@ -437,26 +438,9 @@ init_winExtDLL(void)
             ext_dll_info->pfSnmpExtensionInit(uptime_reference,
                                               &ext_dll_info->
                                               subagentTrapEvent, &view);
-
-        if (!result) {
-            DEBUGMSG(("winExtDLL",
-                      "init_winExtDLL: initialization of DLL %s failed.\n",
-                      ext_dll_info->dll_name));
-            /*
-             * At least on Windows 7 SnmpExtensionInit() in some extension
-             * agent DLLs returns "FALSE" although initialization
-             * succeeded. Hence ignore the SnmpExtensionInit() return value on
-             * Windows Vista and later.
-             */
-            if (s_versioninfo.dwMajorVersion < 6) {
-                snmp_log(LOG_ERR,
-                         "init_winExtDLL: initialization of DLL %s failed.\n",
-                         ext_dll_info->dll_name);
-                FreeLibrary(ext_dll_info->dll_handle);
-                ext_dll_info->dll_handle = 0;
-                continue;
-            }
-        }
+        DEBUGMSG(("winExtDLL",
+                  "init_winExtDLL: DLL %s initialization result %d\n",
+                  ext_dll_info->dll_name, result));
 
         if (ext_dll_info->subagentTrapEvent != NULL) {
             xarray_push_back(&s_trapevent,
@@ -466,35 +450,39 @@ init_winExtDLL(void)
 
         memset(&ext_dll_view_info, 0, sizeof(ext_dll_view_info));
         ext_dll_view_info.winextdll_info = ext_dll_info;
-        if (view.idLength == 0) {
-            DEBUGMSG(("winExtDLL",
-                      "init_winExtDLL: DLL %s did not register an OID range.\n",
-                      ext_dll_info->dll_name));
-            continue;
+        DEBUGMSG(("winExtDLL",
+                  "init_winExtDLL: DLL %s view length %d\n",
+                  ext_dll_info->dll_name, view.idLength));
+        if (view.idLength) {
+            /*
+             * Skip the mib-2 system section on Windows Vista and later because
+             * at least on a 64-bit Windows 7 system all queries in that section
+             * fail with status "generic error".
+             */
+            if (s_versioninfo.dwMajorVersion >= 6
+                && snmp_oid_compare_w_n(view.ids, view.idLength,
+                                        mibii_system_mib,
+                                        sizeof(mibii_system_mib) /
+                                        sizeof(mibii_system_mib[0])) == 0) {
+                DEBUGMSG(("winExtDLL",
+                          "init_winExtDLL: skipping system section of DLL %s.\n",
+                          ext_dll_info->dll_name));
+            } else {
+                copy_oid_n_w(ext_dll_view_info.name,
+                             &ext_dll_view_info.name_length,
+                             view.ids, view.idLength);
+                xarray_push_back(&s_winextdll_view, &ext_dll_view_info);
+            }
         }
-        /*
-         * Skip the mib-2 system section on Windows Vista and later because
-         * at least on a 64-bit Windows 7 system all queries in that section
-         * fail with status "generic error".
-         */
-        if (s_versioninfo.dwMajorVersion >= 6
-            && snmp_oid_compare_w_n(view.ids, view.idLength, mibii_system_mib,
-                                    sizeof(mibii_system_mib) /
-                                    sizeof(mibii_system_mib[0])) == 0) {
-            DEBUGMSG(("winExtDLL",
-                      "init_winExtDLL: skipping system section of DLL %s.\n",
-                      ext_dll_info->dll_name));
-            continue;
-        }
-        copy_oid_n_w(ext_dll_view_info.name, &ext_dll_view_info.name_length,
-                     view.ids, view.idLength);
-        xarray_push_back(&s_winextdll_view, &ext_dll_view_info);
 
         /*
          * Loop looking for more supported views. 
          */
         while (ext_dll_info->pfSnmpExtensionInitEx
                && ext_dll_info->pfSnmpExtensionInitEx(&view)) {
+            DEBUGMSG(("winExtDLL",
+                      "init_winExtDLL: DLL %s view length %d\n",
+                      ext_dll_info->dll_name, view.idLength));
             memset(&ext_dll_view_info, 0, sizeof(ext_dll_view_info));
             ext_dll_view_info.winextdll_info = ext_dll_info;
             copy_oid_n_w(ext_dll_view_info.name,
