@@ -138,19 +138,22 @@ main(int argc, char *argv[])
     int             arg;
     int             count;
     int             status;
-    int             exitval = 0;
+    int             exitval = 1;
+
+    SOCK_STARTUP;
 
     /*
      * get the common command line arguments 
      */
     switch (arg = snmp_parse_args(argc, argv, &session, "C:", optProc)) {
     case NETSNMP_PARSE_ARGS_ERROR:
-        exit(1);
+        goto out;
     case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
-        exit(0);
+        exitval = 0;
+        goto out;
     case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -158,7 +161,7 @@ main(int argc, char *argv[])
     names = argc - arg;
     if (names < non_repeaters) {
         fprintf(stderr, "snmpbulkget: need more objects than <nonrep>\n");
-        exit(1);
+        goto out;
     }
 
     namep = name = (struct nameStruct *) calloc(names, sizeof(*name));
@@ -167,14 +170,12 @@ main(int argc, char *argv[])
         if (snmp_parse_oid(argv[arg], namep->name, &namep->name_len) ==
             NULL) {
             snmp_perror(argv[arg]);
-            exit(1);
+            goto out;
         }
         arg++;
         namep++;
     }
 
-    SOCK_STARTUP;
-
     /*
      * open an SNMP session 
      */
@@ -184,10 +185,11 @@ main(int argc, char *argv[])
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmpbulkget", &session);
-        SOCK_CLEANUP;
-        exit(1);
+        goto out;
     }
 
+    exitval = 0;
+
     /*
      * create PDU for GETBULK request and add object name to request 
      */
@@ -245,6 +247,8 @@ main(int argc, char *argv[])
         snmp_free_pdu(response);
 
     snmp_close(ss);
+
+out:
     SOCK_CLEANUP;
     return exitval;
 }
