@@ -186,7 +186,9 @@ main(int argc, char *argv[])
     int             running;
     int             status = STAT_ERROR;
     int             check;
-    int             exitval = 0;
+    int             exitval = 1;
+
+    SOCK_STARTUP;
 
     netsnmp_ds_register_config(ASN_BOOLEAN, "snmpwalk", "includeRequested",
 			       NETSNMP_DS_APPLICATION_ID, 
@@ -203,12 +205,13 @@ main(int argc, char *argv[])
      */
     switch (arg = snmp_parse_args(argc, argv, &session, "C:", optProc)) {
     case NETSNMP_PARSE_ARGS_ERROR:
-        exit(1);
+        goto out;
     case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
-        exit(0);
+        exitval = 0;
+        goto out;
     case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -223,7 +226,7 @@ main(int argc, char *argv[])
         rootlen = MAX_OID_LEN;
         if (snmp_parse_oid(argv[arg], root, &rootlen) == NULL) {
             snmp_perror(argv[arg]);
-            exit(1);
+            goto out;
         }
     } else {
         /*
@@ -233,8 +236,6 @@ main(int argc, char *argv[])
         rootlen = sizeof(objid_mib) / sizeof(oid);
     }
 
-    SOCK_STARTUP;
-
     /*
      * open an SNMP session 
      */
@@ -244,8 +245,7 @@ main(int argc, char *argv[])
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmpbulkwalk", &session);
-        SOCK_CLEANUP;
-        exit(1);
+        goto out;
     }
 
     /*
@@ -263,6 +263,8 @@ main(int argc, char *argv[])
         snmp_get_and_print(ss, root, rootlen);
     }
 
+    exitval = 0;
+
     while (running) {
         /*
          * create PDU for GETBULK request and add object name to request 
@@ -304,6 +306,7 @@ main(int argc, char *argv[])
                             && snmp_oid_compare(name, name_length,
                                                 vars->name,
                                                 vars->name_length) >= 0) {
+                            fflush(stdout);
                             fprintf(stderr, "Error: OID not increasing: ");
                             fprint_objid(stderr, name, name_length);
                             fprintf(stderr, " >= ");
@@ -381,6 +384,7 @@ main(int argc, char *argv[])
         printf("Variables found: %d\n", numprinted);
     }
 
+out:
     SOCK_CLEANUP;
     return exitval;
 }
