@@ -36,6 +36,9 @@
  * DAMAGE.
  */
 
+/* For getlogin() on MinGW */
+#define _POSIX
+
 #include <net-snmp/net-snmp-config.h>
 
 #include <signal.h>
@@ -53,6 +56,12 @@
 #if HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#if HAVE_IO_H
+#include <io.h>
+#endif
 
 #include <math.h>
 
@@ -202,6 +211,11 @@ add_var(netsnmp_pdu *pdu, const char *mibnodename,
         exit(1);
     }
 
+    if (base_length + indexlen > sizeof(base) / sizeof(base[0])) {
+        fprintf(stderr, "internal error for %s, giving up\n", mibnodename);
+        exit(1);
+    }
+
     if (index && indexlen) {
         memcpy(&(base[base_length]), index, indexlen * sizeof(oid));
         base_length += indexlen;
@@ -557,7 +571,7 @@ out:
     return 0;
 }
 
-#ifdef WIN32
+#ifndef HAVE_GETLOGIN
 /* To do: port this function to the Win32 platform. */
 const char *getlogin(void)
 {
