@@ -93,7 +93,7 @@ static size_t   name_length;
 static oid      root[MAX_OID_LEN];
 static size_t   rootlen;
 static int      localdebug;
-static int      exitval = 0;
+static int      exitval = 1;
 static int      use_getbulk = 1;
 static int      max_getbulk = 10;
 static int      extra_columns = 0;
@@ -250,6 +250,8 @@ main(int argc, char *argv[])
     netsnmp_session session, *ss;
     int            total_entries = 0;
 
+    SOCK_STARTUP;
+
     netsnmp_set_line_buffering(stdout);
 
     netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -260,12 +262,13 @@ main(int argc, char *argv[])
      */
     switch (snmp_parse_args(argc, argv, &session, "C:", optProc)) {
     case NETSNMP_PARSE_ARGS_ERROR:
-        exit(1);
+        goto out;
     case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
-        exit(0);
+        exitval = 0;
+        goto out;
     case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -279,13 +282,13 @@ main(int argc, char *argv[])
     if (optind + 1 != argc) {
         fprintf(stderr, "Must have exactly one table name\n");
         usage();
-        exit(1);
+        goto out;
     }
 
     rootlen = MAX_OID_LEN;
     if (!snmp_parse_oid(argv[optind], root, &rootlen)) {
         snmp_perror(argv[optind]);
-        exit(1);
+        goto out;
     }
     localdebug = netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
                                         NETSNMP_DS_LIB_DUMP_PACKET);
@@ -296,15 +299,13 @@ main(int argc, char *argv[])
     /*
      * open an SNMP session 
      */
-    SOCK_STARTUP;
     ss = snmp_open(&session);
     if (ss == NULL) {
         /*
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmptable", &session);
-        SOCK_CLEANUP;
-        exit(1);
+        goto out;
     }
 
 #ifndef NETSNMP_DISABLE_SNMPV1
@@ -312,6 +313,8 @@ main(int argc, char *argv[])
         use_getbulk = 0;
 #endif
 
+    exitval = 0;
+
     do {
         entries = 0;
         allocated = 0;
@@ -322,21 +325,25 @@ main(int argc, char *argv[])
                 get_table_entries(ss);
         }
 
-        if (exitval) {
-            snmp_close(ss);
-            SOCK_CLEANUP;
-            return exitval;
-        }
+        if (exitval)
+            goto close_session;
 
         if (entries || headers_only)
             print_table();
 
         if (data) {
+            int i, j;
+            for (i = 0; i < entries; i++)
+                for (j = 0; j < fields; j++)
+                free(data[i*fields+j]);
             free (data);
             data = NULL;
         }
 
         if (indices) {
+            int i;
+            for (i = 0; i < entries; i++)
+                free(indices[i]);
             free (indices);
             indices = NULL;
         }
@@ -345,15 +352,19 @@ main(int argc, char *argv[])
 
     } while (!end_of_table);
 
-    snmp_close(ss);
-    SOCK_CLEANUP;
-
     if (total_entries == 0)
         printf("%s: No entries\n", table_name);
     if (extra_columns)
 	printf("%s: WARNING: More columns on agent than in MIB\n", table_name);
 
-    return 0;
+    exitval = 0;
+
+close_session:
+    snmp_close(ss);
+
+out:
+    SOCK_CLEANUP;
+    return exitval;
 }
 
 void
@@ -450,6 +461,8 @@ print_table(void)
     }
 
     first_pass = 0;
+    if (index_fmt)
+        free(index_fmt);
 }
 
 void
@@ -696,7 +709,6 @@ get_table_entries(netsnmp_session * ss)
                              vars->name_length)) {
                             break;
                         }
-                        i = vars->name_length - rootlen + 1;
                         if (localdebug || show_index) {
                             if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
                                               NETSNMP_DS_LIB_EXTENDED_INDEX)) {
@@ -740,6 +752,14 @@ get_table_entries(netsnmp_session * ss)
                     if (localdebug && buf) {
                         printf("%s => taken\n", buf);
                     }
+                    if (dp[col]) {
+                        fprintf(stderr, "OID not increasing: %s\n", buf);
+                        running = 0;
+                        vars = NULL;
+                        end_of_table = 1;
+                        exitval = 2;
+                        break;
+                    }
                     out_len = 0;
                     sprint_realloc_value((u_char **)&buf, &buf_len, &out_len, 1,
                                          vars->name, vars->name_length,
@@ -757,6 +777,11 @@ get_table_entries(netsnmp_session * ss)
                         column[col].width = i;
                     }
                 }
+                if (buf) {
+                    free(buf);
+                    buf = NULL;
+                    buf_len = 0;
+                }
 
                 if (end_of_table) {
                     --entries;
@@ -764,9 +789,9 @@ get_table_entries(netsnmp_session * ss)
                      * not part of this subtree 
                      */
                     if (localdebug) {
-                        printf("End of table: %s\n",
-                               buf ? (char *) buf : "[NIL]");
+                        printf("End of table\n");
                     }
+                    snmp_free_pdu(response);
                     running = 0;
                     continue;
                 }
@@ -877,6 +902,10 @@ getbulk_table_entries(netsnmp_session * ss)
                     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
                                               NETSNMP_DS_LIB_EXTENDED_INDEX)) {
                         name_p = strchr(buf, '[');
+                        if (name_p == NULL) {
+                            running = 0;
+                            break;
+                        }
                     } else {
                         switch (netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
                                                   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT)) {
@@ -947,6 +976,13 @@ getbulk_table_entries(netsnmp_session * ss)
                             index_width = i;
                     }
                     dp = data + row * fields;
+                    if (dp[col]) {
+                        fprintf(stderr, "OID not increasing: %s\n", buf);
+                        exitval = 2;
+                        end_of_table = 1;
+                        running = 0;
+                        break;
+                    }
                     out_len = 0;
                     sprint_realloc_value((u_char **)&buf, &buf_len, &out_len, 1,
                                          vars->name, vars->name_length,
@@ -968,6 +1004,11 @@ getbulk_table_entries(netsnmp_session * ss)
                     memcpy(name, last_var->name,
                            name_length * sizeof(oid));
                 }
+                if (buf) {
+                    free(buf);
+                    buf = NULL;
+                    buf_len = 0;
+                }
             } else {
                 /*
                  * error in response, print it 
