@@ -35,9 +35,6 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -53,10 +50,10 @@
 #include <net-snmp/library/callback.h>
 #include <net-snmp/library/snmp_api.h>
 
-netsnmp_feature_child_of(callbacks_all, libnetsnmp)
+netsnmp_feature_child_of(callbacks_all, libnetsnmp);
 
-netsnmp_feature_child_of(callback_count, callbacks_all)
-netsnmp_feature_child_of(callback_list, callbacks_all)
+netsnmp_feature_child_of(callback_count, callbacks_all);
+netsnmp_feature_child_of(callback_list, callbacks_all);
 
 /*
  * the inline callback methods use major/minor to index into arrays.
@@ -121,7 +118,7 @@ NETSNMP_STATIC_INLINE int
 _callback_lock(int major, int minor, const char* warn, int do_assert)
 {
     int lock_holded=0;
-    struct timeval lock_time = { 0, 1000 };
+    NETSNMP_SELECT_TIMEVAL lock_time = { 0, 1000 };
 
 #ifdef NETSNMP_PARANOID_LEVEL_HIGH
     if (major >= MAX_CALLBACK_IDS || minor >= MAX_CALLBACK_SUBIDS) {
