@@ -35,9 +35,6 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
 
 #ifdef HAVE_PRIORITYNAMES
 #include <sys/syslog.h>
@@ -84,15 +81,15 @@ debug_indent_get(void)
 const char*
 debug_indent(void)
 {
-#define SPACES "                                        " \
-               "                                        "
+    static const char SPACES[] = "                                        "
+        "                                        ";
+
     if ((sizeof(SPACES) - 1) < (unsigned int)debugindent) {
         snmp_log(LOG_ERR, "Too deep indentation for debug_indent. "
                  "Consider using \"%%*s\", debug_indent_get(), \"\" instead.");
         return SPACES;
     }
-    return SPACES + sizeof(SPACES) - 1 - debugindent;
-#undef SPACES
+    return &SPACES[sizeof(SPACES) - 1 - debugindent];
 }
 
 void
@@ -203,7 +200,7 @@ debug_register_tokens(const char *tokens)
             } else if (debug_num_tokens < MAX_DEBUG_TOKENS) {
                 if ('-' == *cp) {
                     ++cp;
-                    status = SNMP_DEBUG_EXCLUDED;
+                    status = SNMP_DEBUG_DISABLED;
                 }
                 else
                     status = SNMP_DEBUG_ACTIVE;
