@@ -11,6 +11,11 @@
  * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 /** @defgroup snmp_logging generic logging for net-snmp 
  *  @ingroup library
@@ -18,6 +23,7 @@
  *  @{
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <stdio.h>
 #if HAVE_MALLOC_H
 #include <malloc.h>
@@ -57,8 +63,8 @@
 
 #include <stdarg.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
+#if HAVE_UNISTD_H
+#include <unistd.h>
 #endif
 
 #include <net-snmp/types.h>
@@ -68,7 +74,8 @@
 #include <net-snmp/utilities.h>
 
 #include <net-snmp/library/callback.h>
-#define LOGLENGTH 1024
+
+#include "snmp_syslog.h"
 
 #ifdef va_copy
 #define NEED_VA_END_AFTER_VA_COPY
@@ -80,6 +87,25 @@
 #define va_copy(dest, src) memcpy (&dest, &src, sizeof (va_list))
 #endif
 #endif
+#ifndef HAVE_VSNPRINTF
+#include "snprintf.h"
+#endif
+
+netsnmp_feature_child_of(logging_all, libnetsnmp);
+
+netsnmp_feature_child_of(logging_outputs, logging_all);
+netsnmp_feature_child_of(logging_file, logging_outputs);
+netsnmp_feature_child_of(logging_stdio, logging_outputs);
+netsnmp_feature_child_of(logging_syslog, logging_outputs);
+netsnmp_feature_child_of(logging_external, logging_all);
+
+netsnmp_feature_child_of(enable_stderrlog, logging_all);
+
+netsnmp_feature_child_of(logging_enable_calllog, netsnmp_unused);
+netsnmp_feature_child_of(logging_enable_loghandler, netsnmp_unused);
+
+/* default to the file/stdio/syslog set */
+netsnmp_feature_want(logging_outputs);
 
 /*
  * logh_head:  A list of all log handlers, in increasing order of priority
@@ -89,7 +115,20 @@ netsnmp_log_handler *logh_head = NULL;
 netsnmp_log_handler *logh_priorities[LOG_DEBUG+1];
 static int  logh_enabled = 0;
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
 static char syslogname[64] = DEFAULT_LOG_ID;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
+
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
+netsnmp_log_handler *
+netsnmp_register_stdio_loghandler(int is_stdout, int priority, int priority_max,
+                                  const char *tok);
+#endif
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
+netsnmp_log_handler *
+netsnmp_register_filelog_handler(const char* logfilename, int priority,
+                                 int priority_max, int dont_zero_log);
+#endif
 
 void
 netsnmp_disable_this_loghandler(netsnmp_log_handler *logh)
@@ -110,16 +149,10 @@ netsnmp_enable_this_loghandler(netsnmp_log_handler *logh)
     ++logh_enabled;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
 void
 netsnmp_enable_filelog(netsnmp_log_handler *logh, int dont_zero_log);
-
-#ifndef HAVE_VSNPRINTF
-                /*
-                 * Need to use the UCD-provided one 
-                 */
-int             vsnprintf(char *str, size_t count, const char *fmt,
-                          va_list arg);
-#endif
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
 
 void
 parse_config_logOption(const char *token, char *cptr)
@@ -148,46 +181,6 @@ shutdown_snmp_logging(void)
       netsnmp_remove_loghandler( logh_head );
 }
 
-/*
- * These definitions handle 4.2 systems without additional syslog facilities.
- */
-#ifndef LOG_CONS
-#define LOG_CONS	0       /* Don't bother if not defined... */
-#endif
-#ifndef LOG_PID
-#define LOG_PID		0       /* Don't bother if not defined... */
-#endif
-#ifndef LOG_LOCAL0
-#define LOG_LOCAL0	0
-#endif
-#ifndef LOG_LOCAL1
-#define LOG_LOCAL1	0
-#endif
-#ifndef LOG_LOCAL2
-#define LOG_LOCAL2	0
-#endif
-#ifndef LOG_LOCAL3
-#define LOG_LOCAL3	0
-#endif
-#ifndef LOG_LOCAL4
-#define LOG_LOCAL4	0
-#endif
-#ifndef LOG_LOCAL5
-#define LOG_LOCAL5	0
-#endif
-#ifndef LOG_LOCAL6
-#define LOG_LOCAL6	0
-#endif
-#ifndef LOG_LOCAL7
-#define LOG_LOCAL7	0
-#endif
-#ifndef LOG_DAEMON
-#define LOG_DAEMON	0
-#endif
-#ifndef LOG_USER
-#define LOG_USER	0
-#endif
-
 /* Set line buffering mode for a stream. */
 void
 netsnmp_set_line_buffering(FILE *stream)
@@ -214,7 +207,7 @@ netsnmp_set_line_buffering(FILE *stream)
  *                 OUT - points to last character after the decoded priority
  * @param pri_max - OUT - maximum priority (i.e. 0x7 from "0-7")
  */
-int
+static int
 decode_priority( char **optarg, int *pri_max )
 {
     int pri_low = LOG_DEBUG;
@@ -282,7 +275,8 @@ decode_priority( char **optarg, int *pri_max )
     return pri_low;
 }
 
-int
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
+static int
 decode_facility( char *optarg )
 {
     if (optarg == NULL)
@@ -316,6 +310,7 @@ decode_facility( char *optarg )
         return -1;
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
 int
 snmp_log_options(char *optarg, int argc, char *const *argv)
@@ -372,6 +367,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
     DEBUGMSGT(("logging:options", "*cp: '%c'\n", *cp));
     switch (*cp) {
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
     /*
      * Log to Standard Error
      */
@@ -382,12 +378,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
             optind++;
         /* Fallthrough */
     case 'e':
-        logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_STDERR, priority);
-        if (logh) {
-            netsnmp_set_line_buffering(stderr);
-            logh->pri_max = pri_max;
-            logh->token   = strdup("stderr");
-	}
+        logh = netsnmp_register_stdio_loghandler(0, priority, pri_max, "stderr");
         break;
 
     /*
@@ -400,23 +391,21 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
             optind++;
         /* Fallthrough */
     case 'o':
-        logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_STDERR, priority);
-        if (logh) {
-            netsnmp_set_line_buffering(stdout);
-            logh->pri_max = pri_max;
-            logh->token   = strdup("stdout");
-            logh->imagic  = 1;	    /* stdout, not stderr */
-	}
+        logh = netsnmp_register_stdio_loghandler( 1, priority, pri_max, "stdout" );
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
 
     /*
      * Log to a named file
      */
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
     case 'F':
         priority = decode_priority( &optarg, &pri_max );
-        if (priority == -1 || !argv)  return -1;
-        optarg = argv[++optind];
-        /* Fallthrough */
+        if (priority == -1) return -1;
+        while (*optarg == ' ') optarg++;
+        if (!*optarg && !argv) return -1;
+        else if (!*optarg) optarg = argv[++optind];
+        /* FALL THROUGH */
     case 'f':
         if (inc_optind)
             optind++;
@@ -424,16 +413,13 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
             fprintf(stderr, "Missing log file\n");
             return -1;
         }
-        logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_FILE, priority);
-        if (logh) {
-            logh->pri_max = pri_max;
-            logh->token   = strdup(optarg);
-            netsnmp_enable_filelog(logh,
-                                   netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
-                                                          NETSNMP_DS_LIB_APPEND_LOGFILES));
-	}
+        DEBUGMSGTL(("logging:options", "%d-%d: '%s'\n", priority, pri_max, optarg));
+        logh = netsnmp_register_filelog_handler(optarg, priority, pri_max,
+                                                   -1);
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
     /*
      * Log to syslog
      */
@@ -458,13 +444,17 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
         logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_SYSLOG, priority);
         if (logh) {
             int facility = decode_facility(optarg);
-            if (facility == -1)  return -1;
+            if (facility == -1) {
+                netsnmp_remove_loghandler(logh);
+                return -1;
+            }
             logh->pri_max = pri_max;
             logh->token   = strdup(snmp_log_syslogname(NULL));
             logh->magic   = (void *)(intptr_t)facility;
 	    snmp_enable_syslog_ident(snmp_log_syslogname(NULL), facility);
 	}
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
     /*
      * Don't log 
@@ -496,28 +486,38 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
     return 0;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
 char *
 snmp_log_syslogname(const char *pstr)
 {
   if (pstr)
-    strncpy (syslogname, pstr, sizeof(syslogname));
+    strlcpy (syslogname, pstr, sizeof(syslogname));
 
   return syslogname;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
 void
 snmp_log_options_usage(const char *lead, FILE * outf)
 {
     const char *pri1_msg = " for level 'pri' and above";
     const char *pri2_msg = " for levels 'p1' to 'p2'";
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
     fprintf(outf, "%se:           log to standard error\n", lead);
     fprintf(outf, "%so:           log to standard output\n", lead);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
     fprintf(outf, "%sn:           don't log at all\n", lead);
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
     fprintf(outf, "%sf file:      log to the specified file\n", lead);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
     fprintf(outf, "%ss facility:  log to syslog (via the specified facility)\n", lead);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
     fprintf(outf, "\n%s(variants)\n", lead);
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
     fprintf(outf, "%s[EON] pri:   log to standard error, output or /dev/null%s\n", lead, pri1_msg);
     fprintf(outf, "%s[EON] p1-p2: log to standard error, output or /dev/null%s\n", lead, pri2_msg);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
     fprintf(outf, "%s[FS] pri token:    log to file/syslog%s\n", lead, pri1_msg);
     fprintf(outf, "%s[FS] p1-p2 token:  log to file/syslog%s\n", lead, pri2_msg);
 }
@@ -552,6 +552,7 @@ sprintf_stamp(time_t * now, char *sbuf)
     return sbuf;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
 void
 snmp_disable_syslog_entry(netsnmp_log_handler *logh)
 {
@@ -581,7 +582,9 @@ snmp_disable_syslog(void)
         if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG)
             snmp_disable_syslog_entry(logh);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
 void
 snmp_disable_filelog_entry(netsnmp_log_handler *logh)
 {
@@ -605,6 +608,30 @@ snmp_disable_filelog(void)
         if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_FILE)
             snmp_disable_filelog_entry(logh);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
+
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
+
+netsnmp_log_handler *
+netsnmp_register_stdio_loghandler(int is_stdout, int priority, int priority_max,
+                                  const char *tok)
+{
+    netsnmp_log_handler *logh =
+        netsnmp_register_loghandler(NETSNMP_LOGHANDLER_STDERR, priority);
+    if (NULL == logh) {
+        return NULL;
+    }
+    if (is_stdout) {
+        netsnmp_set_line_buffering(stdout);
+        logh->imagic = 1; /* stdout, not stderr */
+    } else
+        netsnmp_set_line_buffering(stderr);
+
+    logh->pri_max = priority_max;
+    if (tok)
+        logh->token   = strdup(tok);
+    return logh;
+}
 
 /*
  * returns that status of stderr logging
@@ -637,7 +664,9 @@ snmp_disable_stderrlog(void)
             netsnmp_disable_this_loghandler(logh);
 	}
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_CALLLOG
 void
 snmp_disable_calllog(void)
 {
@@ -648,6 +677,7 @@ snmp_disable_calllog(void)
             netsnmp_disable_this_loghandler(logh);
 	}
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_CALLLOG */
 
 void
 snmp_disable_log(void)
@@ -655,10 +685,14 @@ snmp_disable_log(void)
     netsnmp_log_handler *logh;
 
     for (logh = logh_head; logh; logh = logh->next) {
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
         if (logh->type == NETSNMP_LOGHANDLER_SYSLOG)
             snmp_disable_syslog_entry(logh);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
         if (logh->type == NETSNMP_LOGHANDLER_FILE)
             snmp_disable_filelog_entry(logh);
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
         netsnmp_disable_this_loghandler(logh);
     }
 }
@@ -671,15 +705,22 @@ void
 netsnmp_logging_restart(void)
 {
     netsnmp_log_handler *logh;
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
+    int doneone = 0;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
     for (logh = logh_head; logh; logh = logh->next) {
         if (0 == logh->enabled)
             continue;
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
         if (logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
             snmp_disable_syslog_entry(logh);
             snmp_enable_syslog_ident(logh->token,(int)(intptr_t)logh->magic);
+            doneone = 1;
         }
-        else if (logh->type == NETSNMP_LOGHANDLER_FILE) {
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
+        if (logh->type == NETSNMP_LOGHANDLER_FILE && !doneone) {
             snmp_disable_filelog_entry(logh);
             /** hmm, don't zero status isn't saved.. i think it's
              * safer not to overwrite, in case a hup is just to
@@ -687,11 +728,13 @@ netsnmp_logging_restart(void)
              */
             netsnmp_enable_filelog(logh, 1);
         }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
     }
 }
 
 /* ================================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
 void
 snmp_enable_syslog(void)
 {
@@ -710,7 +753,7 @@ snmp_enable_syslog_ident(const char *ident, const int facility)
     void                *eventlog_h = NULL;
 #endif
 
-    snmp_disable_syslog();	/* ??? */
+    snmp_disable_syslog();     /* only one syslog at a time */
 #ifdef WIN32
     eventlog_h = OpenEventLog(NULL, ident);
     if (eventlog_h == NULL) {
@@ -718,8 +761,8 @@ snmp_enable_syslog_ident(const char *ident, const int facility)
 	     * Hmmm.....
 	     * Maybe disable this handler, and log the error ?
 	     */
-        fprintf(stderr, "Could not open event log for %s. "
-                "Last error: 0x%x\n", ident, GetLastError());
+        fprintf(stderr, "Could not open event log for %s. Last error: %u\n",
+                ident, (unsigned int)GetLastError());
         enable = 0;
     }
 #else
@@ -751,7 +794,9 @@ snmp_enable_syslog_ident(const char *ident, const int facility)
         }
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
 void
 netsnmp_enable_filelog(netsnmp_log_handler *logh, int dont_zero_log)
 {
@@ -772,6 +817,24 @@ netsnmp_enable_filelog(netsnmp_log_handler *logh, int dont_zero_log)
     netsnmp_enable_this_loghandler(logh);
 }
 
+netsnmp_log_handler *
+netsnmp_register_filelog_handler(const char* logfilename, int priority,
+                                 int priority_max, int dont_zero_log)
+{
+    netsnmp_log_handler *logh =
+        netsnmp_register_loghandler(NETSNMP_LOGHANDLER_FILE,
+                                    priority );
+    if (NULL == logh)
+        return NULL;
+    logh->pri_max = priority_max;
+    logh->token = strdup(logfilename);
+    if (-1 == dont_zero_log)
+        dont_zero_log = netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                               NETSNMP_DS_LIB_APPEND_LOGFILES);
+    netsnmp_enable_filelog(logh, dont_zero_log);
+    return logh;
+}
+
 void
 snmp_enable_filelog(const char *logfilename, int dont_zero_log)
 {
@@ -787,13 +850,10 @@ snmp_enable_filelog(const char *logfilename, int dont_zero_log)
 
     if (logfilename) {
         logh = netsnmp_find_loghandler( logfilename );
-        if (!logh) {
-            logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_FILE,
-                                               LOG_DEBUG );
-            if (logh)
-                logh->token = strdup(logfilename);
-	}
-        if (logh)
+        if (!logh)
+            logh = netsnmp_register_filelog_handler( logfilename, LOG_DEBUG,
+                                                     0, dont_zero_log );
+        else
             netsnmp_enable_filelog(logh, dont_zero_log);
     } else {
         for (logh = logh_head; logh; logh = logh->next)
@@ -801,8 +861,11 @@ snmp_enable_filelog(const char *logfilename, int dont_zero_log)
                 netsnmp_enable_filelog(logh, dont_zero_log);
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_ENABLE_STDERRLOG
+/* used in the perl modules and ip-mib/ipv4InterfaceTable/ipv4InterfaceTable_subagent.c */
 void
 snmp_enable_stderrlog(void)
 {
@@ -823,8 +886,9 @@ snmp_enable_stderrlog(void)
             logh->token    = strdup("stderr");
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_ENABLE_STDERRLOG */
 
-
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_CALLLOG
 void
 snmp_enable_calllog(void)	/* XXX - or take a callback routine ??? */
 {
@@ -844,7 +908,7 @@ snmp_enable_calllog(void)	/* XXX - or take a callback routine ??? */
             logh->token    = strdup("callback");
     }
 }
-
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_CALLLOG */
 
 
 /* ==================================================== */
@@ -924,23 +988,35 @@ netsnmp_register_loghandler( int type, int priority )
 
     DEBUGMSGT(("logging:register", "registering log type %d with pri %d\n",
                type, priority));
+    if (priority > LOG_DEBUG) {
+        DEBUGMSGT(("logging:register", "  limiting pri %d to %d\n", priority,
+                   LOG_DEBUG));
+        priority = LOG_DEBUG;
+    }
 
     logh->type     = type;
     switch ( type ) {
     case NETSNMP_LOGHANDLER_STDOUT:
         logh->imagic  = 1;
-        /* fallthrough */
+        logh->handler = log_handler_stdouterr;
+        break;
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
     case NETSNMP_LOGHANDLER_STDERR:
         logh->handler = log_handler_stdouterr;
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
     case NETSNMP_LOGHANDLER_FILE:
         logh->handler = log_handler_file;
         logh->imagic  = 1;
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
     case NETSNMP_LOGHANDLER_SYSLOG:
         logh->handler = log_handler_syslog;
         break;
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
     case NETSNMP_LOGHANDLER_CALLBACK:
         logh->handler = log_handler_callback;
         break;
@@ -958,6 +1034,7 @@ netsnmp_register_loghandler( int type, int priority )
 }
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_LOGHANDLER
 int
 netsnmp_enable_loghandler( const char *token )
 {
@@ -982,6 +1059,7 @@ netsnmp_disable_loghandler( const char *token )
     netsnmp_disable_this_loghandler(logh);
     return 1;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_ENABLE_LOGHANDLER */
 
 int
 netsnmp_remove_loghandler( netsnmp_log_handler *logh )
@@ -999,7 +1077,8 @@ netsnmp_remove_loghandler( netsnmp_log_handler *logh )
         logh->next->prev = logh->prev;
 
     for (i=LOG_EMERG; i<=logh->priority; i++)
-        logh_priorities[i] = NULL;
+        if (logh == logh_priorities[i])
+            logh_priorities[i] = logh->next;
     free(NETSNMP_REMOVE_CONST(char*, logh->token));
     SNMP_FREE(logh);
 
@@ -1008,6 +1087,7 @@ netsnmp_remove_loghandler( netsnmp_log_handler *logh )
 
 /* ==================================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
 int
 log_handler_stdouterr(  netsnmp_log_handler* logh, int pri, const char *str)
 {
@@ -1035,8 +1115,10 @@ log_handler_stdouterr(  netsnmp_log_handler* logh, int pri, const char *str)
 
     return 1;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG
 #ifdef WIN32
 int
 log_handler_syslog(  netsnmp_log_handler* logh, int pri, const char *str)
@@ -1088,8 +1170,8 @@ log_handler_syslog(  netsnmp_log_handler* logh, int pri, const char *str)
 	     * Hmmm.....
 	     * Maybe disable this handler, and log the error ?
 	     */
-        fprintf(stderr, "Could not report event.  Last error: 0x%x\n",
-			GetLastError());
+        fprintf(stderr, "Could not report event.  Last error: %u\n",
+                (unsigned int)GetLastError());
         return 0;
     }
     return 1;
@@ -1120,13 +1202,16 @@ log_handler_syslog(  netsnmp_log_handler* logh, int pri, const char *str)
     return 1;
 }
 #endif /* !WIN32 */
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_SYSLOG */
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_FILE
 int
 log_handler_file(    netsnmp_log_handler* logh, int pri, const char *str)
 {
     FILE           *fhandle;
     char            sbuf[40];
+    int             len = strlen( str );
 
     /*
      * We use imagic to save information about whether the next output
@@ -1155,9 +1240,14 @@ log_handler_file(    netsnmp_log_handler* logh, int pri, const char *str)
     }
     fprintf(fhandle, "%s%s", sbuf, str);
     fflush(fhandle);
-    logh->imagic = str[strlen(str) - 1] == '\n';
+    if (len > 0) {
+        logh->imagic = str[len - 1] == '\n';
+    } else {
+        logh->imagic = 0;
+    }
     return 1;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_FILE */
 
 int
 log_handler_callback(netsnmp_log_handler* logh, int pri, const char *str)
@@ -1192,10 +1282,12 @@ log_handler_null(    netsnmp_log_handler* logh, int pri, const char *str)
 void
 snmp_log_string(int priority, const char *str)
 {
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
     static int stderr_enabled = 0;
     static netsnmp_log_handler lh = { 1, 0, 0, 0, "stderr",
                                       log_handler_stdouterr, 0, NULL,  NULL,
                                       NULL };
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
     netsnmp_log_handler *logh;
 
     /*
@@ -1203,26 +1295,30 @@ snmp_log_string(int priority, const char *str)
      * If you don't want stderr logging, then enable something else.
      */
     if (0 == logh_enabled) {
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
         if (!stderr_enabled) {
             ++stderr_enabled;
             netsnmp_set_line_buffering(stderr);
-            log_handler_stdouterr( &lh, LOG_WARNING,
-                                   "No log handling enabled - using stderr logging\n");
         }
         log_handler_stdouterr( &lh, priority, str );
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
 
         return;
     }
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_STDIO
     else if (stderr_enabled) {
         stderr_enabled = 0;
         log_handler_stdouterr( &lh, LOG_INFO,
                                "Log handling defined - disabling stderr\n" );
     }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_STDIO */
         
 
     /*
      * Start at the given priority, and work "upwards"....
      */
+    if (priority > LOG_DEBUG)
+        priority = LOG_DEBUG;
     logh = logh_priorities[priority];
     for ( ; logh; logh = logh->next ) {
         /*
@@ -1273,53 +1369,17 @@ snmp_log_string(int priority, const char *str)
 int
 snmp_vlog(int priority, const char *format, va_list ap)
 {
-    char            buffer[LOGLENGTH];
+    char           *buffer = NULL;
     int             length;
-    char           *dynamic;
-    va_list         aq;
 
-    va_copy(aq, ap);
-    length = vsnprintf(buffer, LOGLENGTH, format, ap);
-    va_end(ap);
-
-    if (length == 0) {
-#ifdef NEED_VA_END_AFTER_VA_COPY
-        va_end(aq);
-#endif
-        return (0);             /* Empty string */
-    }
-
-    if (length == -1) {
+    length = vasprintf(&buffer, format, ap);
+    if (length < 0) {
         snmp_log_string(LOG_ERR, "Could not format log-string\n");
-#ifdef NEED_VA_END_AFTER_VA_COPY
-        va_end(aq);
-#endif
-        return (-1);
+        return -1;
     }
 
-    if (length < LOGLENGTH) {
-        snmp_log_string(priority, buffer);
-#ifdef NEED_VA_END_AFTER_VA_COPY
-        va_end(aq);
-#endif
-        return (0);
-    }
-
-    dynamic = (char *) malloc(length + 1);
-    if (dynamic == NULL) {
-        snmp_log_string(LOG_ERR,
-                        "Could not allocate memory for log-message\n");
-        snmp_log_string(priority, buffer);
-#ifdef NEED_VA_END_AFTER_VA_COPY
-        va_end(aq);
-#endif
-        return (-2);
-    }
-
-    vsnprintf(dynamic, length + 1, format, aq);
-    snmp_log_string(priority, dynamic);
-    free(dynamic);
-    va_end(aq);
+    snmp_log_string(priority, buffer);
+    free(buffer);
     return 0;
 }
 
@@ -1361,11 +1421,13 @@ snmp_log_perror(const char *s)
     }
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_LOGGING_EXTERNAL
 /* external access to logh_head variable */
 netsnmp_log_handler  *
 get_logh_head(void)
 {
 	return logh_head;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LOGGING_EXTERNAL */
 
 /**  @} */
