@@ -1,5 +1,14 @@
 /*
  * snmp_openssl.c
+ *
+ * Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -8,14 +17,78 @@
 
 #include <net-snmp/net-snmp-features.h>
 
+/** OpenSSL compat functions for apps */
+#if defined(NETSNMP_USE_OPENSSL)
+
+#include <string.h>
+#include <openssl/dh.h>
+
+#ifndef HAVE_DH_GET0_PQG
+void
+DH_get0_pqg(const DH *dh, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g)
+{
+   if (p != NULL)
+       *p = dh->p;
+   if (q != NULL)
+       *q = dh->q;
+   if (g != NULL)
+       *g = dh->g;
+}
+#endif
+
+#ifndef HAVE_DH_GET0_KEY
+void
+DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key)
+{
+   if (pub_key != NULL)
+       *pub_key = dh->pub_key;
+   if (priv_key != NULL)
+       *priv_key = dh->priv_key;
+}
+#endif
+
+#ifndef HAVE_DH_SET0_PQG
+int
+DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
+{
+   /* If the fields p and g in d are NULL, the corresponding input
+    * parameters MUST be non-NULL.  q may remain NULL.
+    */
+   if ((dh->p == NULL && p == NULL)
+       || (dh->g == NULL && g == NULL))
+       return 0;
+
+   if (p != NULL) {
+       BN_free(dh->p);
+       dh->p = p;
+   }
+   if (q != NULL) {
+       BN_free(dh->q);
+       dh->q = q;
+   }
+   if (g != NULL) {
+       BN_free(dh->g);
+       dh->g = g;
+   }
+
+   if (q != NULL) {
+       dh->length = BN_num_bits(q);
+   }
+
+   return 1;
+}
+#endif
+#endif /* defined(NETSNMP_USE_OPENSSL) */
+
+/** TLS/DTLS certificatte support */
 #if defined(NETSNMP_USE_OPENSSL) && defined(HAVE_LIBSSL) && !defined(NETSNMP_FEATURE_REMOVE_CERT_UTIL)
 
-netsnmp_feature_require(container_free_all)
+netsnmp_feature_require(container_free_all);
 
-netsnmp_feature_child_of(openssl_cert_get_subjectAltNames, netsnmp_unused)
-netsnmp_feature_child_of(openssl_ht2nid, netsnmp_unused)
-netsnmp_feature_child_of(openssl_err_log, netsnmp_unused)
-netsnmp_feature_child_of(cert_dump_names, netsnmp_unused)
+netsnmp_feature_child_of(openssl_cert_get_subjectAltNames, netsnmp_unused);
+netsnmp_feature_child_of(openssl_ht2nid, netsnmp_unused);
+netsnmp_feature_child_of(openssl_err_log, netsnmp_unused);
+netsnmp_feature_child_of(cert_dump_names, netsnmp_unused);
 
 #include <ctype.h>
 
@@ -47,8 +120,12 @@ void netsnmp_init_openssl(void) {
     DEBUGMSGTL(("snmp_openssl", "initializing\n"));
 
     /* Initializing OpenSSL */
+#ifdef HAVE_SSL_LIBRARY_INIT
     SSL_library_init();
+#endif
+#ifdef HAVE_SSL_LOAD_ERROR_STRINGS
     SSL_load_error_strings();
+#endif
     ERR_load_BIO_strings();
     OpenSSL_add_all_algorithms();
 }
@@ -141,6 +218,7 @@ netsnmp_openssl_cert_get_commonName(X509 *ocert, char **buf, int *len)
 }
 
 #ifndef NETSNMP_FEATURE_REMOVE_CERT_DUMP_NAMES
+
 /** netsnmp_openssl_cert_dump_name: dump subject names in cert
  */
 void
@@ -148,6 +226,7 @@ netsnmp_openssl_cert_dump_names(X509 *ocert)
 {
     int              i, onid;
     X509_NAME_ENTRY *oname_entry;
+    ASN1_STRING     *oname_value;
     X509_NAME       *osubj_name;
     const char      *prefix_short, *prefix_long;
 
@@ -163,12 +242,13 @@ netsnmp_openssl_cert_dump_names(X509 *ocert)
     for (i = 0; i < X509_NAME_entry_count(osubj_name); i++) {
         oname_entry = X509_NAME_get_entry(osubj_name, i);
         netsnmp_assert(NULL != oname_entry);
+        oname_value = X509_NAME_ENTRY_get_data(oname_entry);
 
-        if (oname_entry->value->type != V_ASN1_PRINTABLESTRING)
+        if (oname_value->type != V_ASN1_PRINTABLESTRING)
             continue;
 
         /** get NID */
-        onid = OBJ_obj2nid(oname_entry->object);
+        onid = OBJ_obj2nid(X509_NAME_ENTRY_get_object(oname_entry));
         if (onid == NID_undef) {
             prefix_long = prefix_short = "UNKNOWN";
         }
@@ -179,9 +259,9 @@ netsnmp_openssl_cert_dump_names(X509 *ocert)
 
         DEBUGMSGT(("9:cert:dump:names",
                    "[%02d] NID type %d, ASN type %d\n", i, onid,
-                   oname_entry->value->type));
+                   oname_value->type));
         DEBUGMSGT(("9:cert:dump:names", "%s/%s: '%s'\n", prefix_long,
-                   prefix_short, ASN1_STRING_data(oname_entry->value)));
+                   prefix_short, ASN1_STRING_get0_data(oname_value)));
     }
 }
 #endif /* NETSNMP_FEATURE_REMOVE_CERT_DUMP_NAMES */
@@ -303,6 +383,7 @@ _cert_get_extension_id(X509 *ocert, int which, char **buf, int *len, int flags)
     return _cert_get_extension_at(ocert, pos, buf, len, flags);
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_OPENSSL_CERT_GET_SUBJECTALTNAMES
 /** _cert_get_extension_id_str: get extension field from cert
  * @internal
  */
@@ -325,6 +406,7 @@ _cert_get_extension_id_str(X509 *ocert, int which, char **buf, int *len,
 
     return _cert_get_extension_str_at(ocert, pos, buf, len, flags);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_OPENSSL_CERT_GET_SUBJECTALTNAMES */
 
 static char *
 _extract_oname(const GENERAL_NAME *oname)
@@ -470,7 +552,7 @@ netsnmp_openssl_cert_get_hash_type(X509 *ocert)
     if (NULL == ocert)
         return 0;
 
-    return _nid2ht(OBJ_obj2nid(ocert->sig_alg->algorithm));
+    return _nid2ht(X509_get_signature_nid(ocert));
 }
 
 /**
@@ -487,7 +569,7 @@ netsnmp_openssl_cert_get_fingerprint(X509 *ocert, int alg)
     if (NULL == ocert)
         return NULL;
 
-    nid = OBJ_obj2nid(ocert->sig_alg->algorithm);
+    nid = X509_get_signature_nid(ocert);
     DEBUGMSGT(("9:openssl:fingerprint", "alg %d, cert nid %d (%d)\n", alg, nid,
                _nid2ht(nid)));
         
@@ -502,7 +584,7 @@ netsnmp_openssl_cert_get_fingerprint(X509 *ocert, int alg)
         
         case NS_HASH_NONE:
             snmp_log(LOG_ERR, "hash type none not supported. using SHA1\n");
-            /** fall through */
+            /* FALLTHROUGH */
 
         case NS_HASH_SHA1:
             digest = EVP_sha1();
@@ -615,8 +697,7 @@ netsnmp_openssl_get_cert_chain(SSL *ssl)
         DEBUGMSGT(("ssl:cert:chain", "examining cert chain\n"));
         for(i = 0; i < sk_num((const void *)ochain); ++i) {
             ocert_tmp = (X509*)sk_value((const void *)ochain,i);
-            fingerprint = netsnmp_openssl_cert_get_fingerprint(ocert_tmp,
-                                                               NS_HASH_SHA1);
+            fingerprint = netsnmp_openssl_cert_get_fingerprint(ocert_tmp, -1);
             if (NULL == fingerprint)
                 break;
             cert_map = netsnmp_cert_map_alloc(NULL, ocert);
@@ -706,15 +787,16 @@ _cert_get_san_type(X509 *ocert, int mapType)
         if (GEN_DNS == oname->type) {
             if ((TSNM_tlstmCertSANDNSName == mapType) ||
                 (TSNM_tlstmCertSANAny == mapType)) {
-                lower = buf = _extract_oname( oname );;
+                lower = buf = _extract_oname( oname );
                 break;
             }
         }
         else if (GEN_IPADD == oname->type) {
             if ((TSNM_tlstmCertSANIpAddress == mapType) ||
-                (TSNM_tlstmCertSANAny == mapType))
+                (TSNM_tlstmCertSANAny == mapType)) {
                 buf = _extract_oname(oname);
                 break;
+            }
         }
         else if (GEN_EMAIL == oname->type) {
             if ((TSNM_tlstmCertSANRFC822Name == mapType) ||
@@ -876,4 +958,50 @@ netsnmp_openssl_null_checks(SSL *ssl, int *null_auth, int *null_cipher)
     }
 }
 
+#ifndef HAVE_X509_GET_SIGNATURE_NID
+int X509_get_signature_nid(const X509 *x)
+{
+    return OBJ_obj2nid(x->sig_alg->algorithm);
+}
+#endif
+
+#ifndef HAVE_ASN1_STRING_GET0_DATA
+const unsigned char *ASN1_STRING_get0_data(const ASN1_STRING *x)
+{
+    return x->data;
+}
+#endif
+
+#ifndef HAVE_X509_NAME_ENTRY_GET_OBJECT
+ASN1_OBJECT *X509_NAME_ENTRY_get_object(const X509_NAME_ENTRY *ne)
+{
+    if (ne == NULL)
+        return NULL;
+    return ne->object;
+}
+#endif
+
+#ifndef HAVE_X509_NAME_ENTRY_GET_DATA
+ASN1_STRING *X509_NAME_ENTRY_get_data(const X509_NAME_ENTRY *ne)
+{
+    if (ne == NULL)
+        return NULL;
+    return ne->value;
+}
+#endif
+
+#ifndef HAVE_TLS_METHOD
+const SSL_METHOD *TLS_method(void)
+{
+    return TLSv1_method();
+}
+#endif
+
+#ifndef HAVE_DTLS_METHOD
+const SSL_METHOD *DTLS_method(void)
+{
+    return DTLSv1_method();
+}
+#endif
+
 #endif /* NETSNMP_USE_OPENSSL && HAVE_LIBSSL && !defined(NETSNMP_FEATURE_REMOVE_CERT_UTIL) */
