@@ -4,6 +4,7 @@
 #include <net-snmp/net-snmp-config.h>
 
 #include <net-snmp/types.h>
+#include <net-snmp/library/snmpIPBaseDomain.h>
 #include <net-snmp/library/snmpIPv4BaseDomain.h>
 #include <net-snmp/library/snmp_assert.h>
 
@@ -56,122 +57,69 @@ int
 netsnmp_sockaddr_in2(struct sockaddr_in *addr,
                      const char *inpeername, const char *default_target)
 {
+    struct netsnmp_ep ep;
     int ret;
 
-    if (addr == NULL) {
+    ret = netsnmp_sockaddr_in3(&ep, inpeername, default_target);
+    if (ret == 0)
+        return 0;
+    *addr = ep.a.sin;
+    return ret;
+}
+
+int
+netsnmp_sockaddr_in3(struct netsnmp_ep *ep,
+                     const char *inpeername, const char *default_target)
+{
+    struct sockaddr_in *addr = &ep->a.sin;
+    struct netsnmp_ep_str ep_str;
+    int port, ret;
+
+    if (!ep)
         return 0;
-    }
 
     DEBUGMSGTL(("netsnmp_sockaddr_in",
                 "addr %p, inpeername \"%s\", default_target \"%s\"\n",
-                addr, inpeername ? inpeername : "[NIL]",
+                ep, inpeername ? inpeername : "[NIL]",
                 default_target ? default_target : "[NIL]"));
 
-    memset(addr, 0, sizeof(struct sockaddr_in));
+    memset(ep, 0, sizeof(*ep));
     addr->sin_addr.s_addr = htonl(INADDR_ANY);
     addr->sin_family = AF_INET;
     addr->sin_port = htons((u_short)SNMP_PORT);
 
-    {
-	int port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
-				      NETSNMP_DS_LIB_DEFAULT_PORT);
+    memset(&ep_str, 0, sizeof(ep_str));
+    port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+                              NETSNMP_DS_LIB_DEFAULT_PORT);
+    if (port != 0)
+        snprintf(ep_str.port, sizeof(ep_str.port), "%d", port);
+    else if (default_target &&
+             !netsnmp_parse_ep_str(&ep_str, default_target))
+            snmp_log(LOG_ERR, "Invalid default target %s\n",
+                     default_target);
+    if (inpeername && *inpeername != '\0' &&
+        !netsnmp_parse_ep_str(&ep_str, inpeername))
+        return 0;
 
-	if (port != 0) {
-	    addr->sin_port = htons((u_short)port);
-	} else if (default_target != NULL)
-	    netsnmp_sockaddr_in2(addr, default_target, NULL);
-    }
-
-    if (inpeername != NULL && *inpeername != '\0') {
-	const char     *host, *port;
-	char           *peername = NULL;
-        char           *cp;
+    if (ep_str.port[0])
+        addr->sin_port = htons(atoi(ep_str.port));
+    if (ep_str.iface[0])
+        strlcpy(ep->iface, ep_str.iface, sizeof(ep->iface));
+    if (strcmp(ep_str.addr, "255.255.255.255") == 0) {
         /*
-         * Duplicate the peername because we might want to mank around with
-         * it.
+         * The explicit broadcast address hack
          */
-
-        peername = strdup(inpeername);
-        if (peername == NULL) {
+        DEBUGMSGTL(("netsnmp_sockaddr_in", "Explicit UDP broadcast\n"));
+        addr->sin_addr.s_addr = INADDR_NONE;
+    } else if (strcmp(ep_str.addr, "") != 0) {
+        ret = netsnmp_gethostbyname_v4(ep_str.addr, &addr->sin_addr.s_addr);
+        if (ret < 0) {
+            DEBUGMSGTL(("netsnmp_sockaddr_in",
+                        "couldn't resolve hostname \"%s\"\n", ep_str.addr));
             return 0;
         }
-
-        /*
-         * Try and extract an appended port number.
-         */
-        cp = strchr(peername, ':');
-        if (cp != NULL) {
-            *cp = '\0';
-            port = cp + 1;
-            host = peername;
-        } else {
-            host = NULL;
-            port = peername;
-        }
-
-        /*
-         * Try to convert the user port specifier
-         */
-        if (port && *port == '\0')
-            port = NULL;
-
-        if (port != NULL) {
-            long int l;
-            char* ep;
-
-            DEBUGMSGTL(("netsnmp_sockaddr_in", "check user service %s\n",
-                        port));
-
-            l = strtol(port, &ep, 10);
-            if (ep != port && *ep == '\0' && 0 <= l && l <= 0x0ffff)
-                addr->sin_port = htons((u_short)l);
-            else {
-                if (host == NULL) {
-                    DEBUGMSGTL(("netsnmp_sockaddr_in",
-                                "servname not numeric, "
-				"check if it really is a destination)\n"));
-                    host = port;
-                    port = NULL;
-                } else {
-                    DEBUGMSGTL(("netsnmp_sockaddr_in",
-                                "servname not numeric\n"));
-                    free(peername);
-                    return 0;
-                }
-            }
-        }
-
-        /*
-         * Try to convert the user host specifier
-         */
-        if (host && *host == '\0')
-            host = NULL;
-
-        if (host != NULL) {
-            DEBUGMSGTL(("netsnmp_sockaddr_in",
-                        "check destination %s\n", host));
-
-
-            if (strcmp(peername, "255.255.255.255") == 0 ) {
-                /*
-                 * The explicit broadcast address hack
-                 */
-                DEBUGMSGTL(("netsnmp_sockaddr_in", "Explicit UDP broadcast\n"));
-                addr->sin_addr.s_addr = INADDR_NONE;
-            } else {
-                ret =
-                    netsnmp_gethostbyname_v4(peername, &addr->sin_addr.s_addr);
-                if (ret < 0) {
-                    DEBUGMSGTL(("netsnmp_sockaddr_in",
-                                "couldn't resolve hostname\n"));
-                    free(peername);
-                    return 0;
-                }
-                DEBUGMSGTL(("netsnmp_sockaddr_in",
-                            "hostname (resolved okay)\n"));
-            }
-        }
-	free(peername);
+        DEBUGMSGTL(("netsnmp_sockaddr_in",
+                    "hostname (resolved okay)\n"));
     }
 
     /*
