@@ -41,9 +41,6 @@
 #include <fcntl.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
 #include <pwd.h>
 
 #ifndef MAXPATHLEN
@@ -62,7 +59,7 @@
 #include <net-snmp/library/snmpSocketBaseDomain.h>
 #include <net-snmp/library/read_config.h>
 
-netsnmp_feature_require(user_information)
+netsnmp_feature_require(user_information);
 
 #define MAX_NAME_LENGTH 127
 
@@ -983,7 +980,6 @@ netsnmp_ssh_ctor(void)
     sshDomain.prefix = (const char **)calloc(2, sizeof(char *));
     sshDomain.prefix[0] = "ssh";
 
-    sshDomain.f_create_from_tstring     = NULL;
     sshDomain.f_create_from_tstring_new = netsnmp_ssh_create_tstring;
     sshDomain.f_create_from_ostring     = netsnmp_ssh_create_ostring;
 
