@@ -17,6 +17,7 @@
 #include <net-snmp/net-snmp-config.h>
 
 #include <net-snmp/types.h>
+#include <net-snmp/library/snmpIPBaseDomain.h>
 #include <net-snmp/library/snmpUDPDomain.h>
 #include <net-snmp/library/snmpUDPIPv4BaseDomain.h>
 
@@ -53,10 +54,6 @@
 #include <sys/uio.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/types.h>
 #include <net-snmp/output_api.h>
 #include <net-snmp/config_api.h>
@@ -157,11 +154,11 @@ netsnmp_udp_transport_base(netsnmp_transport *t)
  * the remote address to send things to.
  */
 netsnmp_transport *
-netsnmp_udp_transport(const struct sockaddr_in *addr, int local)
+netsnmp_udp_transport(const struct netsnmp_ep *ep, int local)
 {
     netsnmp_transport *t = NULL;
 
-    t = netsnmp_udpipv4base_transport(addr, local);
+    t = netsnmp_udpipv4base_transport(ep, local);
     if (NULL != t) {
         netsnmp_udp_transport_base(t);
     }
@@ -175,13 +172,13 @@ netsnmp_udp_transport(const struct sockaddr_in *addr, int local)
  * to send from.
  */
 netsnmp_transport *
-netsnmp_udp_transport_with_source(const struct sockaddr_in *addr, int local,
-                                  const struct sockaddr_in *src_addr)
+netsnmp_udp_transport_with_source(const struct netsnmp_ep *ep, int local,
+                                  const struct netsnmp_ep *src_addr)
 
 {
     netsnmp_transport *t = NULL;
 
-    t = netsnmp_udpipv4base_transport_with_source(addr, local, src_addr);
+    t = netsnmp_udpipv4base_transport_with_source(ep, local, src_addr);
     if (NULL != t) {
         netsnmp_udp_transport_base(t);
     }
@@ -369,6 +366,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
         mask.s_addr = 0;
         negate = 0;
     } else {
+        char *strmask;
         if (*source == '!') {
             negate = 1;
             sourcep = source + 1;
@@ -378,7 +376,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
         }
 
         /* Split the source/netmask parts */
-        char *strmask = strchr(sourcep, '/');
+        strmask = strchr(sourcep, '/');
         if (strmask != NULL)
             /* Mask given. */
             *strmask++ = '\0';
@@ -606,9 +604,9 @@ netsnmp_transport *
 netsnmp_udp_create_tstring(const char *str, int local,
 			   const char *default_target)
 {
-    struct sockaddr_in addr;
+    struct netsnmp_ep addr;
 
-    if (netsnmp_sockaddr_in2(&addr, str, default_target)) {
+    if (netsnmp_sockaddr_in3(&addr, str, default_target)) {
         return netsnmp_udp_transport(&addr, local);
     } else {
         return NULL;
@@ -629,10 +627,11 @@ netsnmp_udp_create_tspec(netsnmp_tdomain_spec *tspec)
 netsnmp_transport *
 netsnmp_udp_create_ostring(const void *o, size_t o_len, int local)
 {
-    struct sockaddr_in sin;
+    struct netsnmp_ep ep;
 
-    if (netsnmp_ipv4_ostring_to_sockaddr(&sin, o, o_len))
-        return netsnmp_udp_transport(&sin, local);
+    memset(&ep, 0, sizeof(ep));
+    if (netsnmp_ipv4_ostring_to_sockaddr(&ep.a.sin, o, o_len))
+        return netsnmp_udp_transport(&ep, local);
     return NULL;
 }
 
@@ -645,7 +644,6 @@ netsnmp_udp_ctor(void)
     udpDomain.prefix = (const char**)calloc(2, sizeof(char *));
     udpDomain.prefix[0] = "udp";
 
-    udpDomain.f_create_from_tstring     = NULL;
     udpDomain.f_create_from_tstring_new = netsnmp_udp_create_tstring;
     udpDomain.f_create_from_tspec       = netsnmp_udp_create_tspec;
     udpDomain.f_create_from_ostring     = netsnmp_udp_create_ostring;
